import os
import unittest

from biolinkml.generators.pythongen import PythonGenerator
from tests.test_issues import sourcedir


class Issue5084TestCase(unittest.TestCase):
    def test_issue_84(self):
        yaml_fname = os.path.join(sourcedir, 'issue_84.yaml')
        PythonGenerator(yaml_fname).serialize()

    def test_issue_50(self):
        yaml_fname = os.path.join(sourcedir, 'issue_50.yaml')
        PythonGenerator(yaml_fname).serialize()



if __name__ == '__main__':
    unittest.main()



import importlib
import os
import re
import unittest
from contextlib import redirect_stdout
from io import StringIO

from tests.test_notebooks import output_directory


class NotebookTests(unittest.TestCase):
    def eval_test(self, target: str, import_module: str) -> None:
        output = StringIO()
        output_file = os.path.join(output_directory, target)
        with redirect_stdout(output):
            importlib.import_module(import_module)
        with open(output_file, 'w') as f:
            f.write(re.sub(r'# Generation date: .*', r'# Generation date:', output.getvalue()))
        print(f"Output written to {output_file}")

    def test_examples(self):
        self.eval_test('examples.txt', "tests.test_notebooks.input.examples")

    def test_inheritence(self):
        self.eval_test('inheritence.txt', "tests.test_notebooks.input.inheritence")

    def test_distributed_models(self):
        self.eval_test('distributedmodels.txt', "tests.test_notebooks.input.distributedmodels")


if __name__ == '__main__':
    unittest.main()



import os
from typing import Union, TextIO

import click

from biolinkml.meta import ClassDefinition, SlotDefinition, SchemaDefinition
from biolinkml.utils.formatutils import camelcase, lcamelcase
from biolinkml.utils.generator import Generator, shared_arguments


class ProtoGenerator(Generator):
    """
    A `Generator` for creating Protobuf schemas from a biolinkml schema.
    
    """
    generatorname = os.path.basename(__file__)
    generatorversion = "0.1.1"
    valid_formats = ['proto']
    visit_all_class_slots = True

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], **kwargs) -> None:
        super().__init__(schema, **kwargs)
        self.relative_slot_num = 0

    def visit_class(self, cls: ClassDefinition) -> bool:
        if cls.mixin or cls.abstract or not cls.slots:
            return False
        if cls.description:
            for dline in cls.description.split('\n'):
                print(f'// {dline}')
        print(f'message {camelcase(cls.name)}')
        print(" {")
        self.relative_slot_num = 0
        return True

    def end_class(self, cls: ClassDefinition) -> None:
        print(" }")

    def visit_class_slot(self, cls: ClassDefinition, aliased_slot_name: str, slot: SlotDefinition) -> None:
        qual = 'repeated ' if slot.multivalued else 'optional ' if not slot.required or slot.key else ''
        slotname = lcamelcase(aliased_slot_name)
        slot_range = camelcase(slot.range)
        self.relative_slot_num += 1
        print(f"  {qual}{slotname} {slot_range} = {self.relative_slot_num}")


@shared_arguments(ProtoGenerator)
@click.command()
def cli(yamlfile, **args):
    """ Generate proto representation of biolink model """
    print(ProtoGenerator(yamlfile, **args).serialize(**args))



"""Generate Summary Spreadsheets

"""
import os
import sys
from csv import DictWriter
from typing import Union, TextIO, Optional

import click

from biolinkml.meta import ClassDefinition, SchemaDefinition, SlotDefinition
from biolinkml.utils.formatutils import camelcase
from biolinkml.utils.generator import Generator, shared_arguments


class SummaryGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.1.1"
    valid_formats = ['tsv']

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], **args) -> None:
        super().__init__(schema, **args)
        self.dirname = None
        self.classtab: Optional[DictWriter] = None
        self.slottab: Optional[DictWriter] = None
        self.dialect = 'excel-tab'

    def visit_schema(self, **_) -> None:
        self.classtab = DictWriter(sys.stdout,
                                   ['Class Name', 'Parent Class', 'YAML Class Name', 'Description',
                                    'Flags', 'Slot Name', 'YAML Slot Name', 'Range', 'Card', 'Slot Description', 'URI'],
                                   dialect=self.dialect)
        self.classtab.writeheader()

    def visit_class(self, cls: ClassDefinition) -> bool:
        self.classtab.writerow({'Class Name': camelcase(cls.name),
                                'Parent Class': camelcase(cls.is_a) if cls.is_a else '',
                                'YAML Class Name': cls.name,
                                'Description': cls.description})
        return True

    def visit_class_slot(self, cls: ClassDefinition, aliased_slot_name: str, slot: SlotDefinition) -> None:
        min_card = 1 if slot.required else 0
        max_card = "*" if slot.multivalued else 1
        abstract = 'A' if slot.abstract else ''
        key = 'K' if slot.key else ''
        identifier = 'I' if slot.identifier else ''
        readonly = 'R' if slot.readonly else ''
        ref = '*' if slot.range in self.schema.classes and not slot.inlined else ''
        self.classtab.writerow({'Slot Name': aliased_slot_name,
                                'Flags': abstract + key + identifier + readonly,
                                'Card': f"{min_card}..{max_card}",
                                'YAML Slot Name': slot.name if slot.name != aliased_slot_name else '',
                                'Range': ref + self.class_or_type_name(slot.range),
                                'Slot Description': slot.description,
                                'URI': slot.slot_uri})


@shared_arguments(SummaryGenerator)
@click.command()
def cli(yamlfile, **args):
    """ Generate TSV summary files for viewing in Excel and the like """
    print(SummaryGenerator(yamlfile, **args).serialize(**args))



from dataclasses import dataclass, InitVar
from typing import Union, Any, Dict, List

import yaml
import os
from jsonasobj import JsonObj, as_json
from rdflib import Graph
from yaml.constructor import ConstructorError

from biolinkml.utils.context_utils import CONTEXTS_PARAM_TYPE, merge_contexts


class YAMLRoot(JsonObj):

    """
    The root object for all python YAML representations
    """
    def __post_init__(self, **kwargs):
        if kwargs:
            messages: List[str] = []
            for k in kwargs.keys():
                v = repr(kwargs[k])[:40].replace('\n', '\\n')
                messages.append(f"Unknown argument: {k} = {v}  {k.loc() if getattr(k, 'loc', None) else ''}")
            raise ValueError('\n'.join(messages))

    def _default(self, obj):
        """ JSON serializer callback.
        1) Filter out empty values (None, {}, [] and False) and mangle the names
        2) Add ID entries for dictionary entries

        :param obj: YAMLRoot object to serialize
        :return: Serialized version of obj
        """

        if isinstance(obj, JsonObj):
            rval = dict()
            for k, v in obj.__dict__.items():
                is_classvar = k.startswith("type_") and hasattr(type(obj), k)
                if is_classvar:
                    print(f"***** {k} is classvar ")
                if not is_classvar and not k.startswith('_') and v is not None and\
                        (not isinstance(v, (dict, list, bool)) or v):
                    if isinstance(v, dict):
                        itemslist = []
                        for vk, vv in v.items():
                            # if isinstance(vv, ClassDefinition):
                            #     vv['@id'] = camelcase(vk)
                            # elif isinstance(vv, (SlotDefinition, TypeDefinition)):
                            #     if k != 'slot_usage':
                            #         vv['@id'] = underscore(vk)
                            itemslist.append(vv)
                        rval[k] = itemslist
                    else:
                        rval[k] = v
            return rval
        else:
            return super()._default(obj)


def root_representer(dumper: yaml.Dumper, data: YAMLRoot):
    """ YAML callback -- used to filter out empty values (None, {}, [] and false)

    @param dumper: data dumper
    @param data: data to be dumped
    @return:
    """
    rval = dict()
    for k, v in data.__dict__.items():
        if not k.startswith('_') and v is not None and (not isinstance(v, (dict, list)) or v):
            rval[k] = v
    return dumper.represent_data(rval)


def as_yaml(element: YAMLRoot) -> str:
    """
    Return element in a YAML representation

    :param element: YAML object
    :return: Stringified representation
    """
    return yaml.dump(element, Dumper=yaml.SafeDumper, sort_keys=False)


def as_json_object(element: YAMLRoot, contexts: CONTEXTS_PARAM_TYPE = None) -> JsonObj:
    """
    Return the representation of element as a JsonObj object
    :param element: element to return
    :param contexts: context(s) to include in the output
    :return: JsonObj representation of element
    """
    rval = JsonObj(**element.__dict__)
    rval['type'] = element.__class__.__name__
    context_element = merge_contexts(contexts)
    if context_element:
        rval['@context'] = context_element['@context']
    return rval


class TypedNode:
    def __init__(self, v: Union[Any, "TypedNode"]):
        self._s = v._s if isinstance(v, TypedNode) else None
        self._len = v._len if isinstance(v, TypedNode) else None
        super().__init__()

    def add_node(self, node):
        self._s = node.start_mark
        self._len = node.end_mark.index - node.start_mark.index
        return self

    def loc(self) -> str:
        return f"{self._s.name}: line {self._s.line + 1} col {self._s.column + 1}"


class extended_str(str, TypedNode):
    pass


class extended_int(int, TypedNode):
    pass


class extended_float(float, TypedNode):
    pass


class DupCheckYamlLoader(yaml.loader.SafeLoader):
    """
    A YAML loader that throws an error when the same key appears twice
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.add_constructor(yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG, DupCheckYamlLoader.map_constructor)
        self.add_constructor('tag:yaml.org,2002:str', DupCheckYamlLoader.construct_yaml_str)
        self.add_constructor('tag:yaml.org,2002:int', DupCheckYamlLoader.construct_yaml_int)
        self.add_constructor('tag:yaml.org,2002:float', DupCheckYamlLoader.construct_yaml_float)

    def construct_yaml_int(self, node):
        """ Scalar constructor that returns the node information as the value """
        return extended_int(super().construct_yaml_int(node)).add_node(node)

    def construct_yaml_str(self, node):
        """ Scalar constructor that returns the node information as the value """
        return extended_str(super().construct_yaml_str(node)).add_node(node)

    def construct_yaml_float(self, node):
        """ Scalar constructor that returns the node information as the value """
        return extended_float(super().construct_yaml_float(node)).add_node(node)

    @staticmethod
    def map_constructor(loader,  node, deep=False):
        """ Duplicate of constructor.construct_mapping w/ exception that we check for dups

        """
        if not isinstance(node, yaml.MappingNode):
            raise ConstructorError(None, None,
                    "expected a mapping node, but found %s" % node.id,
                    node.start_mark)
        mapping = {}
        for key_node, value_node in node.value:
            key = loader.construct_object(key_node, deep=deep)
            value = loader.construct_object(value_node, deep=deep)
            if key in mapping:
                raise ValueError(f"Duplicate key: \"{key}\"")
            mapping[key] = value
        return mapping


yaml.SafeDumper.add_multi_representer(YAMLRoot, root_representer)
yaml.SafeDumper.add_multi_representer(extended_str, yaml.SafeDumper.represent_str)
yaml.SafeDumper.add_multi_representer(extended_int, yaml.SafeDumper.represent_int)
yaml.SafeDumper.add_multi_representer(extended_float, yaml.SafeDumper.represent_float)
yaml.SafeDumper.add_multi_representer(str, yaml.SafeDumper.represent_str)
yaml.SafeDumper.add_multi_representer(int, yaml.SafeDumper.represent_int)
yaml.SafeDumper.add_multi_representer(float, yaml.SafeDumper.represent_float)


def as_rdf(element: YAMLRoot, contexts: CONTEXTS_PARAM_TYPE = None) -> Graph:
    """
    Convert element into an RDF graph guided by the context(s) in contexts
    :param element: element to represent in RDF
    :param contexts: JSON-LD context(s) in the form of a file or URL name, a json string or a json obj
    :return: rdflib Graph containing element
    """

    jsonld = as_json_object(element, contexts)
    graph = Graph()
    graph.parse(data=as_json(jsonld), format="json-ld", prefix=True)
    return graph





# Auto generated from biolink-model.yaml by namespacegen.py version: 0.4.0
# Generation date:
# Schema: Biolink_Model
#
# id: https://w3id.org/biolink/biolink-model
# description: Entity and association taxonomy and datamodel for life-sciences data
# license: https://creativecommons.org/publicdomain/zero/1.0/

from collections import defaultdict
from typing import Iterable, Dict, Tuple

from biolinkml.utils.curienamespace import CurieNamespace

GENE = 'gene'
DISEASE = 'disease'
CHEMICAL_SUBSTANCE = 'chemical substance'

SYMBOL = 'Approved_Symbol'


class IdentifierResolverException(RuntimeError):
    pass


class BiolinkNameSpace:
    """
    Map of BioLink Model registered URI Namespaces
    """

    _namespaces = [
        CurieNamespace('BFO', 'http://purl.obolibrary.org/obo/BFO_'),
        CurieNamespace('BIOGRID', 'http://thebiogrid.org/'),
        CurieNamespace('BioSample', 'http://example.org/UNKNOWN/BioSample/'),
        CurieNamespace('CAID', 'http://example.org/UNKNOWN/CAID/'),
        CurieNamespace('CHEBI', 'http://purl.obolibrary.org/obo/CHEBI_'),
        CurieNamespace('CHEMBL_COMPOUND', 'http://identifiers.org/chembl.compound/'),
        CurieNamespace('CHEMBL_TARGET', 'http://identifiers.org/chembl.target/'),
        CurieNamespace('CIO', 'http://purl.obolibrary.org/obo/CIO_'),
        CurieNamespace('CIViC', 'http://example.org/UNKNOWN/CIViC/'),
        CurieNamespace('CL', 'http://purl.obolibrary.org/obo/CL_'),
        CurieNamespace('CLO', 'http://purl.obolibrary.org/obo/CLO_'),
        CurieNamespace('ClinVar', 'http://www.ncbi.nlm.nih.gov/clinvar/'),
        CurieNamespace('DBSNP', 'http://identifiers.org/dbsnp/'),
        CurieNamespace('DOID', 'http://purl.obolibrary.org/obo/DOID_'),
        CurieNamespace('DRUGBANK', 'http://identifiers.org/drugbank/'),
        CurieNamespace('ECO', 'http://purl.obolibrary.org/obo/ECO_'),
        CurieNamespace('ECTO', 'http://example.org/UNKNOWN/ECTO/'),
        CurieNamespace('EFO', 'http://purl.obolibrary.org/obo/EFO_'),
        CurieNamespace('ENSEMBL', 'http://ensembl.org/id/'),
        CurieNamespace('ExO', 'http://example.org/UNKNOWN/ExO/'),
        CurieNamespace('FAO', 'http://purl.obolibrary.org/obo/FAO_'),
        CurieNamespace('GENO', 'http://purl.obolibrary.org/obo/GENO_'),
        CurieNamespace('GO', 'http://purl.obolibrary.org/obo/GO_'),
        CurieNamespace('GOLD_META', 'http://identifiers.org/gold.meta/'),
        CurieNamespace('GTOPDB', 'http://example.org/UNKNOWN/GTOPDB/'),
        CurieNamespace('HANCESTRO', 'http://example.org/UNKNOWN/HANCESTRO/'),
        CurieNamespace('HGNC', 'http://www.genenames.org/cgi-bin/gene_symbol_report?hgnc_id='),
        CurieNamespace('HGVS', 'http://example.org/UNKNOWN/HGVS/'),
        CurieNamespace('HMDB', 'http://www.hmdb.ca/metabolites/'),
        CurieNamespace('HP', 'http://purl.obolibrary.org/obo/HP_'),
        CurieNamespace('IAO', 'http://purl.obolibrary.org/obo/IAO_'),
        CurieNamespace('INCHI', 'http://identifiers.org/inchi/'),
        CurieNamespace('INCHIKEY', 'http://identifiers.org/inchikey/'),
        CurieNamespace('IUPHAR', 'http://example.org/UNKNOWN/IUPHAR/'),
        CurieNamespace('IntAct', 'http://example.org/UNKNOWN/IntAct/'),
        CurieNamespace('KEGG', 'http://identifiers.org/kegg/'),
        CurieNamespace('MEDDRA', 'http://purl.bioontology.org/ontology/MEDDRA/'),
        CurieNamespace('MESH', 'http://purl.obolibrary.org/obo/MESH_'),
        CurieNamespace('MGI', 'http://www.informatics.jax.org/accession/MGI:'),
        CurieNamespace('MIR', 'http://identifiers.org/mir/'),
        CurieNamespace('MONDO', 'http://purl.obolibrary.org/obo/MONDO_'),
        CurieNamespace('MYVARIANT_HG19', 'http://example.org/UNKNOWN/MYVARIANT_HG19/'),
        CurieNamespace('MYVARIANT_HG38', 'http://example.org/UNKNOWN/MYVARIANT_HG38/'),
        CurieNamespace('NCBIGene', 'http://www.ncbi.nlm.nih.gov/gene/'),
        CurieNamespace('NCIT', 'http://purl.obolibrary.org/obo/NCIT_'),
        CurieNamespace('OBAN', 'http://purl.org/oban/'),
        CurieNamespace('OBI', 'http://purl.obolibrary.org/obo/OBI_'),
        CurieNamespace('OGMS', 'http://purl.obolibrary.org/obo/OGMS_'),
        CurieNamespace('OIO', 'http://www.geneontology.org/formats/oboInOwl#'),
        CurieNamespace('OMIM', 'http://purl.obolibrary.org/obo/OMIM_'),
        CurieNamespace('ORPHANET', 'http://identifiers.org/orphanet/'),
        CurieNamespace('PANTHER', 'http://www.pantherdb.org/panther/family.do?clsAccession='),
        CurieNamespace('PHARMGKB', 'http://example.org/UNKNOWN/PHARMGKB/'),
        CurieNamespace('PMID', 'http://www.ncbi.nlm.nih.gov/pubmed/'),
        CurieNamespace('PO', 'http://purl.obolibrary.org/obo/PO_'),
        CurieNamespace('PR', 'http://purl.obolibrary.org/obo/PR_'),
        CurieNamespace('PUBCHEM', 'http://example.org/UNKNOWN/PUBCHEM/'),
        CurieNamespace('PW', 'http://purl.obolibrary.org/obo/PW_'),
        CurieNamespace('PomBase', 'https://www.pombase.org/spombe/result/'),
        CurieNamespace('RHEA', 'http://identifiers.org/rhea/'),
        CurieNamespace('RNAcentral', 'http://example.org/UNKNOWN/RNAcentral/'),
        CurieNamespace('RO', 'http://purl.obolibrary.org/obo/RO_'),
        CurieNamespace('Reactome', 'http://example.org/UNKNOWN/Reactome/'),
        CurieNamespace('SEMMEDDB', 'http://example.org/UNKNOWN/SEMMEDDB/'),
        CurieNamespace('SGD', 'https://www.yeastgenome.org/locus/'),
        CurieNamespace('SIO', 'http://semanticscience.org/resource/SIO_'),
        CurieNamespace('SMPDB', 'http://smpdb.ca/view/'),
        CurieNamespace('SO', 'http://purl.obolibrary.org/obo/SO_'),
        CurieNamespace('UBERON', 'http://purl.obolibrary.org/obo/UBERON_'),
        CurieNamespace('UMLS', 'http://linkedlifedata.com/resource/umls/id/'),
        CurieNamespace('UMLSSC', 'https://uts-ws.nlm.nih.gov/rest/semantic-network/semantic-network/current/TUI/'),
        CurieNamespace('UMLSSG', 'https://uts-ws.nlm.nih.gov/rest/semantic-network/semantic-network/current/GROUP/'),
        CurieNamespace('UMLSST', 'https://uts-ws.nlm.nih.gov/rest/semantic-network/semantic-network/current/STY/'),
        CurieNamespace('UNII', 'http://fdasis.nlm.nih.gov/srs/unii/'),
        CurieNamespace('UNIPROTKB', 'http://example.org/UNKNOWN/UNIPROTKB/'),
        CurieNamespace('UO', 'http://purl.obolibrary.org/obo/UO_'),
        CurieNamespace('UPHENO', 'http://purl.obolibrary.org/obo/UPHENO_'),
        CurieNamespace('UniProtKB', 'http://identifiers.org/uniprot/'),
        CurieNamespace('VMC', 'http://example.org/UNKNOWN/VMC/'),
        CurieNamespace('WB', 'http://identifiers.org/wb/'),
        CurieNamespace('WD', 'http://example.org/UNKNOWN/WD/'),
        CurieNamespace('WIKIPATHWAYS', 'http://identifiers.org/wikipathways/'),
        CurieNamespace('ZFIN', 'http://zfin.org/'),
        CurieNamespace('biolink', 'https://w3id.org/biolink/vocab/'),
        CurieNamespace('biolinkml', 'https://w3id.org/biolink/biolinkml/'),
        CurieNamespace('dct', 'http://example.org/UNKNOWN/dct/'),
        CurieNamespace('dcterms', 'http://purl.org/dc/terms/'),
        CurieNamespace('dictyBase', 'http://dictybase.org/gene/'),
        CurieNamespace('faldo', 'http://biohackathon.org/resource/faldo#'),
        CurieNamespace('owl', 'http://www.w3.org/2002/07/owl#'),
        CurieNamespace('pav', 'http://purl.org/pav/'),
        CurieNamespace('qud', 'http://qudt.org/1.1/schema/qudt#'),
        CurieNamespace('rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'),
        CurieNamespace('rdfs', 'http://www.w3.org/2000/01/rdf-schema#'),
        CurieNamespace('skos', 'http://www.w3.org/2004/02/skos/core#'),
        CurieNamespace('void', 'http://rdfs.org/ns/void#'),
        CurieNamespace('wgs', 'http://www.w3.org/2003/01/geo/wgs84_pos'),
        CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#'),
    ]

    # class level dictionaries

    _prefix_map: Dict[str, CurieNamespace] = {}

    @classmethod
    def _get_prefix_map(cls):
        if not cls._prefix_map:
            for ns in cls._namespaces:
                # index by upper case for uniformity of search
                cls._prefix_map[ns.prefix.upper()] = ns
        return cls._prefix_map

    @classmethod
    def parse_curie(cls, curie: str) -> Tuple[CurieNamespace, str]:
        """
        Parse a candidate CURIE
        :param curie: candidate curie string
        :return: CURIE namespace and object_id
        """
        found = CurieNamespace("", ""), curie  # default value if not a CURIE or unknown XMLNS prefix
        if ':' in curie:
            part = curie.split(":")
            # Normalize retrieval with upper case of prefix for lookup
            prefix = part[0].upper()
            if prefix in cls._get_prefix_map():
                found = cls._prefix_map[prefix], part[1]
        return found

    @classmethod
    def parse_uri(cls, uri: str) -> Tuple[CurieNamespace,  str]:
        """
        Parse a candidate URI
        :param uri: candidate URI string
        :return: namespace and object_id
        """
        found = CurieNamespace("", ""), uri   # default value returned if unknown URI namespace

        # TODO: is there a more efficient lookup scheme here than a linear search of namespaces?
        for ns in cls._namespaces:
            base_uri = str(ns)
            if uri.startswith(base_uri):
                # simple minded deletion of base_uri to give the object_id
                object_id = uri.replace(base_uri, "")
                found = ns, object_id
                break
        return found

    @classmethod
    def parse_identifier(cls,  identifier: str) -> Tuple[CurieNamespace,  str]:

        # trivial case of a null identifier?
        if not identifier:
            return CurieNamespace("", ""), ""

        # check if this is a candidate URI...
        if identifier.lower().startswith("http"):
            # guess that perhaps it is, so try to parse it
            return cls.parse_uri(identifier)

        else:  # attempt to parse as a CURIE
            return cls.parse_curie(identifier)


def object_id(identifier, keep_version=False) -> str:
    """
    Returns the core object_id of a CURIE, with or without the version suffix.
    Note:  not designed to be used with a URI (will give an invalid outcome)
    :param identifier: candidate CURIE identifier for processing
    :param keep_version: True if the version string suffix is to be retained in the identifier
    :return:
    """
    # trivial case: null input value?
    if not identifier:
        return identifier

    if ':' in identifier:
        identifier = identifier.split(":")[1]

    if not keep_version and '.' in identifier:
        identifier = identifier.split(".")[0]

    return identifier


def fix_curies(identifiers, prefix=''):
    """
    Applies the specified XMLNS prefix to (an) identifier(s) known
    to be "raw" IDs as keys in a dictionary or elements in a list (or a simple string)
    :param identifiers:
    :param prefix:
    :return:
    """
    if not prefix:
        # return identifiers without modification
        # Caller may already consider them in curie format
        return identifiers

    if isinstance(identifiers, dict):
        curie_dict = defaultdict(dict)
        for key in identifiers.keys():
            curie_dict[prefix + ':' + object_id(key, keep_version=True)] = identifiers[key]
        return curie_dict

    # identifiers assumed to be just a single object identifier
    elif isinstance(identifiers, str):
        # single string to convert
        return prefix + ':' + object_id(identifiers, keep_version=True)

    elif isinstance(identifiers, Iterable):
        return [prefix + ':' + object_id(x, keep_version=True) for x in identifiers]

    else:
        raise RuntimeError("fix_curie() is not sure how to fix an instance of data type '", type(identifiers))


def curie(identifier) -> str:
    # Ignore enpty strings
    if not identifier:
        return ""
    else:
        namespace: CurieNamespace
        identifier_object_id: str
        namespace, identifier_object_id = BiolinkNameSpace.parse_identifier(identifier)
        return namespace.curie(identifier_object_id)



import os
import unittest
from types import ModuleType

from jsonasobj import loads

from biolinkml.generators.jsonldcontextgen import ContextGenerator
from biolinkml.generators.jsonldgen import JSONLDGenerator
from biolinkml.generators.pythongen import PythonGenerator
from biolinkml.utils.yamlutils import as_rdf
from tests.utils.metadata_filters import ldcontext_metadata_filter, metadata_filter, json_metadata_filter
from tests.test_utils import inputdir
from tests.test_utils import outputdir

from tests.utils.generator_utils import GeneratorTestCase

expected_rdf = """
@prefix : <http://example.org/test/uriandcurie> .
@prefix ex: <http://example.org/test/inst#> .
@prefix test: <http://example.org/test/> .
@prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> .
@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .
@prefix xml: <http://www.w3.org/XML/1998/namespace> .
@prefix xsd: <http://www.w3.org/2001/XMLSchema#> .

ex:obj1 a test:uriandcurieC1 ;
    test:uriandcuriehasCurie ex:curie ;
    test:uriandcuriehasNcName "A123" ;
    test:uriandcuriehasURI <http://example.org/test/uri> ;
    test:uriandcurieid2 ex:id2 .
"""


# Note: GeneratorTestCase is the
class URIAndCurieTestCase(GeneratorTestCase):
    source_path: str = inputdir
    target_path: str = outputdir
    model_path: str = inputdir
    model_name: str = 'uriandcurie'
    output_name: str = 'uriandcurie'
    
    def test_uriÎ_and_curie(self):
        """ Compile a model of URI's and Curies and then test the various types """
        self.single_file_generator('py', PythonGenerator, filtr=metadata_filter)

        # Make sure the python is valid
        with open(os.path.join(self.source_path, self.model_name + '.py')) as f:
            model = f.read()
        spec = compile(model, 'test', 'exec')
        module = ModuleType('test')
        exec(spec, module.__dict__)

        # Check that the interpretations are correct
        msg = self.single_file_generator('jsonld', ContextGenerator, filtr=ldcontext_metadata_filter,
                                         fail_if_expected_missing=False)
        msg = self.single_file_generator('json', JSONLDGenerator, filtr=json_metadata_filter,
                                         fail_if_expected_missing=False)
        if msg:
            self.fail(msg)

        curie_obj = module.C1("ex:obj1",
                              hasCurie="ex:curie",
                              hasURI="http://example.org/test/uri",
                              hasNcName="A123",
                              id2="ex:id2")
        instance_jsonld = loads('{ "ex": "http://example.org/test/inst#" }')

        g = as_rdf(curie_obj, [os.path.join(self.source_path, self.model_name + '.jsonld'), instance_jsonld])
        self.rdf_comparator(expected_rdf, g, os.path.join(self.target_path, self.model_name + '.jsonld'))


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from biolinkml import LOCAL_METAMODEL_YAML_FILE, LOCAL_METAMODEL_JSONLD_FILE, LOCAL_METAMODEL_LDCONTEXT_FILE, \
    LOCAL_MAPPING_YAML_FILE, LOCAL_MAPPING_JSONLD_FILE, LOCAL_MAPPING_LDCONTEXT_FILE, \
    LOCAL_TYPES_YAML_FILE, LOCAL_TYPES_JSONLD_FILE, LOCAL_TYPES_LDCONTEXT_FILE, \
    LOCAL_SHEXC_FILE_NAME, LOCAL_SHEXJ_FILE_NAME, LOCAL_RDF_FILE_NAME, \
    METAMODEL_URI, METAMODEL_NAMESPACE_NAME, METAMODEL_NAMESPACE, METATYPE_NAMESPACE_NAME, METATYPE_URI, \
    METATYPE_NAMESPACE, METAMAPPING_URI, METAMAPPING_NAMESPACE_NAME, METAMAPPING_NAMESPACE, METAMODEL_NAME, \
    METATYPE_NAME, METAMAPPING_NAME
from biolinkml.utils.rawloader import load_raw_schema


class ModelURITestCase(unittest.TestCase):
    """ Make sure that the URI's in the include files match what is actually in the output"""

    def validate_yaml_content(self, uri, name, namespace_name, namespace, source) -> None:
        def check_yaml(source_file):
            model_yaml = load_raw_schema(source_file)
            self.assertEqual(uri, model_yaml.id)
            self.assertEqual(name, model_yaml.name)
            self.assertEqual(namespace_name, model_yaml.default_prefix)
            self.assertEqual(namespace, model_yaml.prefixes[model_yaml.default_prefix].prefix_reference)
            self.assertEqual(source_file, model_yaml.source_file)

        check_yaml(source)
        check_yaml(uri)

    def test_model_uris(self):
        """ Test that the variables in meta.yaml match the contents of biolinkml/__init__.py """
        for filename in [LOCAL_RDF_FILE_NAME, LOCAL_SHEXJ_FILE_NAME, LOCAL_SHEXC_FILE_NAME, LOCAL_TYPES_LDCONTEXT_FILE,
                         LOCAL_SHEXC_FILE_NAME, LOCAL_SHEXJ_FILE_NAME, LOCAL_RDF_FILE_NAME, LOCAL_TYPES_LDCONTEXT_FILE,
                         LOCAL_MAPPING_YAML_FILE, LOCAL_MAPPING_LDCONTEXT_FILE, LOCAL_MAPPING_JSONLD_FILE,
                         LOCAL_TYPES_LDCONTEXT_FILE, LOCAL_TYPES_JSONLD_FILE, LOCAL_TYPES_YAML_FILE,
                         LOCAL_MAPPING_JSONLD_FILE, LOCAL_METAMODEL_JSONLD_FILE, LOCAL_METAMODEL_LDCONTEXT_FILE]:
            self.assertTrue(os.path.exists(filename), msg=f"{filename} does not exist")
        self.validate_yaml_content(METAMODEL_URI, METAMODEL_NAME, METAMODEL_NAMESPACE_NAME, METAMODEL_NAMESPACE,
                                   LOCAL_METAMODEL_YAML_FILE)
        self.validate_yaml_content(METATYPE_URI, METATYPE_NAME, METATYPE_NAMESPACE_NAME, METATYPE_NAMESPACE,
                                   LOCAL_TYPES_YAML_FILE)
        self.validate_yaml_content(METAMAPPING_URI, METAMAPPING_NAME, METAMAPPING_NAMESPACE_NAME, METAMAPPING_NAMESPACE,
                                   LOCAL_MAPPING_YAML_FILE)


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from contextlib import redirect_stderr
from io import StringIO

import jsonasobj
from jsonasobj import as_json, load

from biolinkml import LOCAL_METAMODEL_YAML_FILE, LOCAL_TYPES_YAML_FILE
from biolinkml.utils.schemaloader import SchemaLoader
from tests.test_utils import inputdir, outputdir
from tests.test_utils.support.base import Base
from tests.utils.metadata_filters import json_metadata_filter


class SchemaLoaderTestCase(Base):
    def test_basic_merge(self):
        """ Test the basic merge paths """
        errfile = StringIO()
        with redirect_stderr(errfile):
            self.eval_loader("merge1")
        # Note: There is something about the PyCharm / UnitTest package that, if you are running a lot of tests, the
        # output ends up getting redirected to the test runner rather than stderr.  If there is nothing at all, we
        # will assume that this is the case.
        if errfile.getvalue().strip():
            self.assertIn("Shared slot and subset names: s1, s2", errfile.getvalue().strip())
        else:
            print("*** Warning not tested -- stderr redirect isn't working")

    def test_mergeerror1(self):
        """ Test conflicting definitions path """
        fn = os.path.join(inputdir, 'mergeerror1.yaml')
        with self.assertRaises(ValueError):
            SchemaLoader(fn)

    def test_meta(self):
        """ Load up a static image of the metamodel and emit it

        Note: you may want to periodically refresh the metamodel in the data section """
        self.maxDiff = None
        self.eval_loader('meta', source=LOCAL_METAMODEL_YAML_FILE)

    def test_types(self):
        self.eval_loader('includes/types', source=LOCAL_TYPES_YAML_FILE)

    def test_imports(self):
        self.eval_loader('base')

    def test_error_paths(self):
        """ Test various loader error situations"""

        fn = os.path.join(inputdir, 'loadererror1.yaml')
        with self.assertRaises(ValueError, msg="Unknown slot domain should fail") as e:
            SchemaLoader(fn).resolve()
        self.assertIn('loadererror1.yaml: line 11 col 13', str(e.exception))

        fn = os.path.join(inputdir, 'loadererror2.yaml')
        with self.assertRaises(ValueError, msg="Optional key slot should fail") as e:
            SchemaLoader(fn).resolve()
        self.assertIn('loadererror2.yaml: line 10 col 3', str(e.exception))

        fn = os.path.join(inputdir, 'loadertest1.yaml')
        schema = SchemaLoader(fn).resolve()
        self.assertEqual('string', schema.slots['s1'].range)

        fn = os.path.join(inputdir, 'loadererror4.yaml')
        with self.assertRaises(ValueError, msg="Default prefix is not defined") as e:
            SchemaLoader(fn).resolve()
        self.assertIn('loadererror4.yaml: line 6 col 17', str(e.exception))

    def test_empty_range(self):
        """ A type must have either a base or a parent """
        fn = os.path.join(inputdir, 'loadererror5.yaml')
        with self.assertRaises(ValueError, msg="Range error should be raised") as e:
            _ = SchemaLoader(fn).resolve()
        self.assertIn('loadererror5.yaml: line 9 col 3', str(e.exception))

    def test_multi_key(self):
        """ Multiple keys are not supported """
        fn = os.path.join(inputdir, 'loadererror6.yaml')
        with self.assertRaises(ValueError, msg="Two or more keys are not allowed") as e:
            _ = SchemaLoader(fn).resolve()
        self.assertIn('loadererror6.yaml: line 16 col 3', str(e.exception))

        fn = os.path.join(inputdir, 'loadererror7.yaml')
        with self.assertRaises(ValueError, msg="Two or more keys are not allowed") as e:
            _ = SchemaLoader(fn).resolve()
        self.assertIn('loadererror7.yaml: line 17 col 3', str(e.exception))

    @unittest.skipIf(True, "Never impelemented checking key and identifier")
    def test_key_and_id(self):
        """ A slot cannot be both a key and an identifier """
        fn = os.path.join(inputdir, 'loadererror8.yaml')
        _ = SchemaLoader(fn).resolve()
        with self.assertRaises(ValueError, msg="A slot cannot be both a key and identifier") as e:
            _ = SchemaLoader(fn).resolve()
        self.assertIn('loadererror8.yaml', str(e.exception))

        fn = os.path.join(inputdir, 'loadererror9.yaml')
        with self.assertRaises(ValueError, msg="A slot cannot be both a key and identifier") as e:
            _ = SchemaLoader(fn).resolve()
        self.assertIn('loadererror9.yaml', str(e.exception))

    def test_missing_type_uri(self):
        """ A type with neither a typeof or uri is an error """
        fn = os.path.join(inputdir, 'loadererror10.yaml')
        with self.assertRaises(ValueError, msg="A non-typeof type has to have a URI") as e:
            _ = SchemaLoader(fn).resolve()
        self.assertIn('loadererror10.yaml: line 12 col 3', str(e.exception))
        fn = os.path.join(inputdir, 'loaderpass11.yaml')
        _ = SchemaLoader(fn).resolve()

    def test_undefined_subset(self):
        """ Throw an error on an undefined subset reference """
        fn = os.path.join(inputdir, 'loadererror11.yaml')
        with self.assertRaises(ValueError, msg="Subset references must be valid") as e:
            _ = SchemaLoader(fn).resolve()
        self.assertIn('loadererror11.yaml: line 22 col 16', str(e.exception))

    def test_importmap(self):
        """ Test the importmap parameter """
        fn = os.path.join(inputdir, 'import_test_1.yaml')
        importmap = {"http://example.org/import_test_2" : "import_test_2",
                      "loc/imp3": "import_test_3",
                      "base:import_test_4": "http://example.org/import_test_4",
                      "http://example.org/import_test_4": "import_test_4",
                      "types": "http://w3id.org/biolink/biolinkml/types"}
        schema = SchemaLoader(fn, importmap=importmap).resolve()
        schema_image = as_json(schema)
        outfile = os.path.join(outputdir, 'import_test_1.json')
        if not os.path.exists(outfile):
            with open(outfile, 'w') as f:
                f.write(schema_image)
                self.fail("File {outfile} written - rerun test")
        expected = load(outfile)
        self.maxDiff = None
        self.assertEqual(json_metadata_filter(jsonasobj.as_json(expected)), json_metadata_filter(schema_image))



if __name__ == '__main__':
    unittest.main()



"""
Generate GOlr YAML schema definitions.

These can be converted to solr schema-xml, and used in amigo-bbop tools

See the golr-views directory in this repo for examples

"""
import os
from typing import Union, TextIO, List, Optional

import click
from dataclasses import dataclass

from biolinkml.utils.generator import Generator, shared_arguments
from biolinkml.meta import SchemaDefinition, ClassDefinition, SlotDefinition
from biolinkml.utils.metamodelcore import empty_list
from biolinkml.utils.formatutils import underscore
from biolinkml.utils.yamlutils import YAMLRoot, as_yaml


@dataclass
class GOLRField(YAMLRoot):
    id: str
    description: str
    display_name: str
    property: List = empty_list()
    cardinality: Optional[str] = None


@dataclass
class GOLRClass(YAMLRoot):
    id: str
    schema_generating: bool
    description: str
    display_name: str
    document_category: str
    weight: int
    fields: List[GOLRField] = empty_list()


class GolrSchemaGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.1.1"
    valid_formats = ["golr"]
    visit_all_class_slots = True

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], directory: str = None, **kwargs) -> None:
        super().__init__(schema, **kwargs)
        self.dirname: str = directory
        self.class_obj: Optional[GOLRClass] = None

    def visit_schema(self, directory: str, **_) -> None:
        self.dirname = directory
        if directory:
            os.makedirs(directory, exist_ok=True)
        # write_golr_yaml_to_dir(schema, dir)

    def visit_class(self, cls: ClassDefinition) -> bool:
        if not cls.abstract:
            self.class_obj = GOLRClass(id=underscore(cls.name),
                                       schema_generating=True,
                                       description=cls.description,
                                       display_name=cls.name,
                                       document_category=cls.name,
                                       weight=20)
            return True
        else:
            return False

    def end_class(self, cls: ClassDefinition) -> None:
        fn = os.path.join(self.dirname, underscore(cls.name + '-config.yaml'))
        with open(fn, 'w') as f:
            f.write(as_yaml(self.class_obj))

    def visit_class_slot(self, cls: ClassDefinition, aliased_slot_name: str, slot: SlotDefinition) -> None:
        field = GOLRField(id=underscore(aliased_slot_name), description=slot.description, display_name=slot.name)
        if slot.multivalued:
            field.cardinality = 'multi'
        self.class_obj.fields.append(field)


@shared_arguments(GolrSchemaGenerator)
@click.command()
@click.option("--dir", "-d", default='golr-views', help="Output directory")
def cli(yamlfile, dir=None, **args):
    """ Generate GOLR representation of a biolink model """
    print(GolrSchemaGenerator(yamlfile, directory=dir, **args).serialize(directory=dir, **args))



import logging
import os

from biolinkml import LOCAL_METAMODEL_YAML_FILE, LOCAL_METAMODEL_LDCONTEXT_FILE, METAMODEL_FILE_NAME

refresh_files = False               # True means update target files

testdir = os.path.abspath(os.path.dirname(__file__))
sourcedir = os.path.join(testdir, 'source')
targetdir = os.path.join(testdir, 'target')
source_yaml_path = LOCAL_METAMODEL_YAML_FILE
target_yaml_path = os.path.join(targetdir, METAMODEL_FILE_NAME)
source_context_path = LOCAL_METAMODEL_LDCONTEXT_FILE

DO_SHEX_VALIDATION = False

# Set this to True if you are making changes to the model itself.  Note, however, that it needs to be reset to False
# once the new types and/or meta elements have been submitted to the main repository
USE_LOCAL_IMPORT_MAP = False

# There are lots of warnings emitted by the generators. Default logging level
DEFAULT_LOG_LEVEL = logging.ERROR


# Auto generated from .yaml by pythongen.py version:
# Generation date:
# Schema: uriandcurie
#
# id: http://example.org/test/uriandcurie
# description:
# license:

import dataclasses
import sys
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from biolinkml.utils.slot import Slot
from biolinkml.utils.metamodelcore import empty_list, empty_dict, bnode
from biolinkml.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
if sys.version_info < (3, 7, 6):
    from biolinkml.utils.dataclass_extensions_375 import dataclasses_init_fn_with_kwargs
else:
    from biolinkml.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from biolinkml.utils.formatutils import camelcase, underscore, sfx
from rdflib import Namespace, URIRef
from biolinkml.utils.curienamespace import CurieNamespace
from biolinkml.utils.metamodelcore import Curie, ElementIdentifier, NCName, NodeIdentifier, URI, URIorCURIE

metamodel_version = "1.4.3"

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
M = CurieNamespace('m', 'http://example.org/test/uriandcurie')
SHEX = CurieNamespace('shex', 'http://www.w3.org/ns/shex#')
XSD = CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#')
DEFAULT_ = M


# Types
class String(str):
    """ A character string """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "string"
    type_model_uri = M.String


class Uriorcurie(URIorCURIE):
    """ a URI or a CURIE """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uriorcurie"
    type_model_uri = M.Uriorcurie


class Uri(URI):
    """ a complete URI """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "uri"
    type_model_uri = M.Uri


class Curie(Curie):
    """ a CURIE """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "curie"
    type_model_uri = M.Curie


class Ncname(NCName):
    """ Prefix part of CURIE """
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "ncname"
    type_model_uri = M.Ncname


class Objectidentifier(ElementIdentifier):
    """ A URI or CURIE that represents an object in the model. """
    type_class_uri = SHEX.iri
    type_class_curie = "shex:iri"
    type_name = "objectidentifier"
    type_model_uri = M.Objectidentifier


class Nodeidentifier(NodeIdentifier):
    """ A URI, CURIE or BNODE that represents a node in a model. """
    type_class_uri = SHEX.nonliteral
    type_class_curie = "shex:nonliteral"
    type_name = "nodeidentifier"
    type_model_uri = M.Nodeidentifier


# Class references
class C1Id(ElementIdentifier):
    pass


@dataclass
class C1(YAMLRoot):
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = M.C1
    class_class_curie: ClassVar[str] = "m:C1"
    class_name: ClassVar[str] = "c1"
    class_model_uri: ClassVar[URIRef] = M.C1

    id: Union[str, C1Id]
    hasCurie: Optional[Union[str, Curie]] = None
    hasURI: Optional[Union[str, URI]] = None
    hasNcName: Optional[Union[str, NCName]] = None
    id2: Optional[Union[str, NodeIdentifier]] = None

    def __post_init__(self, **kwargs: Dict[str, Any]):
        if self.id is None:
            raise ValueError(f"id must be supplied")
        if not isinstance(self.id, C1Id):
            self.id = C1Id(self.id)
        if self.hasCurie is not None and not isinstance(self.hasCurie, Curie):
            self.hasCurie = Curie(self.hasCurie)
        if self.hasURI is not None and not isinstance(self.hasURI, URI):
            self.hasURI = URI(self.hasURI)
        if self.hasNcName is not None and not isinstance(self.hasNcName, NCName):
            self.hasNcName = NCName(self.hasNcName)
        if self.id2 is not None and not isinstance(self.id2, NodeIdentifier):
            self.id2 = NodeIdentifier(self.id2)
        super().__post_init__(**kwargs)



# Slots
class slots:
    pass

slots.id = Slot(uri=M.id, name="id", curie=M.curie('id'),
                      model_uri=M.id, domain=None, range=URIRef)

slots.hasCurie = Slot(uri=M.hasCurie, name="hasCurie", curie=M.curie('hasCurie'),
                      model_uri=M.hasCurie, domain=None, range=Optional[Union[str, Curie]])

slots.hasURI = Slot(uri=M.hasURI, name="hasURI", curie=M.curie('hasURI'),
                      model_uri=M.hasURI, domain=None, range=Optional[Union[str, URI]])

slots.hasNcName = Slot(uri=M.hasNcName, name="hasNcName", curie=M.curie('hasNcName'),
                      model_uri=M.hasNcName, domain=None, range=Optional[Union[str, NCName]])

slots.id2 = Slot(uri=M.id2, name="id2", curie=M.curie('id2'),
                      model_uri=M.id2, domain=None, range=Optional[Union[str, NodeIdentifier]])


import os
import unittest

from jsonasobj import loads
from rdflib import Namespace

from biolinkml.generators.shexgen import ShExGenerator
from tests.test_issues import sourcedir

DCT = Namespace("http://purl.org/dc/terms/")

class DefinedPrefixTestCase(unittest.TestCase):
    def test_dct_prefix(self):
        """ Make sure prefixes are handled correctly """
        with self.assertRaises(ValueError, msg="A colon in an identifier is illegal"):
            shex = loads(ShExGenerator(os.path.join(sourcedir, 'Issue_6.yaml'), format='json').serialize())
        shex = loads(ShExGenerator(os.path.join(sourcedir, 'Issue_6_fixed.yaml'), format='json').serialize())
        company_shape = [s for s in shex.shapes if 'Company' in s.id][0]
        for expr in company_shape.expression.expressions:
            if expr.min == 0:
                self.assertEqual(str(DCT.created), expr.predicate, )
                return
        self.assertFalse(True, "DCT.created predicate not found")


if __name__ == '__main__':
    unittest.main()



"""
YAML Schema to RDF Generator

Generate a JSON LD representation of the model

"""
import os
from typing import Union, TextIO, Optional

import click
from rdflib import Graph
from rdflib.plugin import plugins as rdflib_plugins, Parser as rdflib_Parser

from biolinkml import METAMODEL_CONTEXT_URI
from biolinkml.generators.jsonldgen import JSONLDGenerator
from biolinkml.meta import SchemaDefinition
from biolinkml.utils.generator import Generator, shared_arguments


class RDFGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.1.1"
    valid_formats =['ttl'] + sorted([x.name for x in rdflib_plugins(None, rdflib_Parser) if '/' not in str(x.name)])
    visit_all_class_slots = False

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], **kwargs) -> None:
        super().__init__(schema, **kwargs)

    def _data(self, g: Graph) -> str:
        return g.serialize(format='turtle' if self.format == 'ttl' else self.format).decode()

    def end_schema(self, output: Optional[str] = None, context: str = METAMODEL_CONTEXT_URI, **_) -> None:
        gen = JSONLDGenerator(self, fmt=JSONLDGenerator.valid_formats[0], emit_metadata=self.emit_metadata,
                              importmap=self.importmap)
        jsonld_str = gen.serialize(context=context)
        graph = Graph()
        graph.parse(data=jsonld_str, format="json-ld", base=self.namespaces._base, prefix=True)
        if output:
            with open(output, 'w') as outf:
                outf.write(self._data(graph))
        else:
            print(self._data(graph))


@shared_arguments(RDFGenerator)
@click.command()
@click.option("-o", "--output", help="Output file name")
@click.option("--context", default=METAMODEL_CONTEXT_URI, multiple=True,
              help=f"JSONLD context file (default: {METAMODEL_CONTEXT_URI})")
def cli(yamlfile, **kwargs):
    """ Generate an RDF representation of a biolink model """
    print(RDFGenerator(yamlfile, **kwargs).serialize(**kwargs))



import os

from tests.utils.dirutils import make_and_clear_directory

testscriptsdir = os.path.abspath(os.path.join(os.path.dirname(__file__)))
testscriptstempdir = os.path.join(testscriptsdir, 'temp')

if not os.path.exists(testscriptstempdir):
    make_and_clear_directory(testscriptstempdir)


import unittest

# This has to occur post ClickTestCase
import click

from biolinkml.generators.graphqlgen import cli
from tests import source_yaml_path
from tests.test_scripts.clicktestcase import ClickTestCase


class GenGraphqlTestCase(ClickTestCase):
    testdir = "gengraphql"
    click_ep = cli
    prog_name = "gen-graphql"

    def test_help(self):
        self.do_test("--help", 'help')

    def test_meta(self):
        self.do_test(source_yaml_path, 'meta.graphql')
        self.do_test(source_yaml_path + ' -f xsv', 'meta_error', error=click.exceptions.BadParameter)


if __name__ == '__main__':
    unittest.main()



import unittest
import sys
import dataclasses
from dataclasses import dataclass, InitVar, field, fields
from typing import Optional, ClassVar, Dict, Any

import yaml

from biolinkml.utils.yamlutils import YAMLRoot, TypedNode, DupCheckYamlLoader

if sys.version_info < (3, 7, 6):
    from biolinkml.utils.dataclass_extensions_375 import dataclasses_init_fn_with_kwargs
else:
    from biolinkml.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs


class Issue83TestCase(unittest.TestCase):
    # The goal is to provide line numbers on error messages.   We've tweaked the parser so that it returns augmented
    # str's and int's with the line numbers on them.  The problem we are trying to address now is that the dataclass
    # constructor doesn't support **argv out of the box.  We can certainly tweak the generator to emit the __init__
    # method to do this, but it would be really handy

    @dataclass
    class FesterBesterTester(YAMLRoot):
        cv: ClassVar[int] = 42

        a: Optional[int] = 0
        b: Optional[str] = None

    def test_initvar(self):
        with self.assertRaises(ValueError) as e:
            Issue83TestCase.FesterBesterTester(a=12, b="Sell", c="buy")
        self.assertEqual("Unknown argument: c = 'buy'", str(e.exception).strip())
        yaml_str = """base:
        a: 17
        b: Ice Cream
        c: sell
"""
        parsed_yaml = yaml.load(yaml_str, DupCheckYamlLoader)
        with self.assertRaises(ValueError) as e:
            Issue83TestCase.FesterBesterTester(**parsed_yaml['base'])
        self.assertEqual("Unknown argument: c = 'sell'  <unicode string>: line 4 col 9",
                         str(e.exception).strip())

        parsed_yaml['base'].pop('c', None)
        try:
            Issue83TestCase.FesterBesterTester(**parsed_yaml['base'])
        except Exception as e:
            self.fail(f'Raised exception unexpectedly: {str(e.exception)}')


if __name__ == '__main__':
    unittest.main()



import builtins
import datetime
import re
from dataclasses import field
from typing import Union, Optional, Tuple
from urllib.parse import urlparse

from rdflib import Literal, BNode, URIRef
from rdflib.namespace import is_ncname
from rdflib.term import Identifier as rdflib_Identifier

from biolinkml.utils.namespaces import Namespaces
from biolinkml.utils.strictness import is_strict


# ===========================
# Fields for use in dataclass
# ===========================
from biolinkml.utils.yamlutils import TypedNode


def empty_list():
    """ Return a field with a list factory """
    return field(default_factory=list)


def empty_dict():
    """ Return a field with a dictionary factory """
    return field(default_factory=dict)

def addl_args():
    rval = field(default_factory=dict)
    rval.name = '**args'
    return rval

def empty_set():
    """" Return a field with a set factory """
    return field(default_factory=set)


def bnode():
    """ Return a field with a BNode factory """
    return field(default_factory=lambda: BNode().n3())


# List of builtins for mangling names
builtinnames = dir(builtins)


class NCName(str):
    """ Wrapper for NCName class """
    def __init__(self, v: str) -> None:
        if is_strict() and not self.is_valid(v):
            raise ValueError(f"{v}: invalid prefix")
        self.nsm: Optional[Namespaces] = None
        super().__init__()

    @classmethod
    def is_valid(cls, v: str) -> bool:
        return is_ncname(v) if v else True      # Empty is default name


class Identifier(str, TypedNode):
    """ A datatype that can be a URI, CURIE or a BNode """
    def __init__(self, v: Union[str, URIRef, BNode, "URIorCURIE", "Identifier"]) -> None:
        v = str(v) if not isinstance(v, str) else v
        if is_strict() and not self.is_valid(v):
            raise ValueError(f"{v}: is not a valid Identifier")
        TypedNode.__init__(self, v)

    @classmethod
    def is_valid(cls, v: Union[str, URIRef, BNode, "URIorCURIE", "Identifier"]) -> bool:
        if v is None:
            return False
        if v.startswith('_:'):
            pfx, ln = v.split(':')
            return len(ln) == 1 and bool(Curie.term_name.match(ln))
        else:
            return URIorCURIE.is_valid(v)

    def as_identifier(self, nsm: Optional[Namespaces]) -> Optional[rdflib_Identifier]:
        if self.startswith('_:'):
            return BNode(self)
        elif URIorCURIE.is_absolute(self):
            return URIRef(self)
        elif nsm:
            return URIorCURIE(self).as_uri(nsm)
        return None


class URIorCURIE(Identifier):
    """ A datatype that can either be a URI or a CURIE """
    def __init__(self, v: Union[str, URIRef, "Curie", "URIorCURIE"], *args, **kwargs) -> None:
        if is_strict() and not URIorCURIE.is_valid(v):
            raise ValueError(f"{v} is not a valid URI or CURIE")
        super().__init__(v, *args, **kwargs)

    @classmethod
    def is_valid(cls, v: Union[str, URIRef, "Curie", "URIorCURIE"]) -> bool:
        v = str(v)
        if ':' in v and '://' not in v:
            return URIorCURIE.is_curie(v)
        else:
            return URI.is_valid(v)

    @staticmethod
    def is_absolute(v: str) -> bool:
        return bool(urlparse(v).netloc)

    @staticmethod
    def is_curie(v: str, nsm: Optional[Namespaces] = None) -> bool:
        if ':' in v and '://' not in v:
            ns, ln = v.split(':', 1)
            return len(ns) == 0 or (NCName.is_valid(ns) and
                                    (nsm is None or any(ns == nsns for nsns, _ in nsm.namespaces())))
        return False

    def as_uri(self, nsm: Namespaces) -> Optional[URIRef]:
        if self.is_absolute(self):
            return URIRef(self)
        return Curie(self).as_uri(nsm)


class URI(URIorCURIE):
    """ A relative absolute URI
    """
    def __init__(self, v: str, *args, **kwargs) -> None:
        if is_strict() and not URI.is_valid(v):
            raise ValueError(f"{v}: is not a valid URI")
        super().__init__(v, *args, **kwargs)

    @classmethod
    def is_valid(cls, v: str) -> bool:
        # TODO: need to get a bit more rigorous here...
        return v is not None and not URIorCURIE.is_curie(v) and bool(urlparse(v))


class Curie(URIorCURIE):

    """ Wrapper for an element that MUST be represented as a CURIE """
    def __init__(self, v: str, *args, **kwargs) -> None:
        if is_strict() and not self.is_valid(v):
            raise ValueError(f"{v} is not a valid CURIE")
        super().__init__(v, *args, **kwargs)

    # TODO: see whether we can leverage the rdflib RDFA termorcurie package
    term_name = re.compile("^[A-Za-z]([A-Za-z0-9._-]|/)*$")

    @classmethod
    def ns_ln(cls, v: str) -> Optional[Tuple[str, str]]:
        # See if this is indeed a valid CURIE, ie, it can be split by a colon
        curie_split = v.split(':', 1)
        if len(curie_split) == 1:
            # there is no ':' character in the string, ie, it is not a valid CURIE
            return None
        else:
            prefix = curie_split[0].lower()
            if not NCName.is_valid(prefix):
                return None
            reference = curie_split[1]
            if not cls.term_name.match(reference):
                return None
        return prefix, reference

    @classmethod
    def is_valid(cls, v: str) -> bool:
        return cls.ns_ln(v) is not None

    # This code was extracted from the termorcurie package of the rdfa
    def as_uri(self, nsm: Namespaces) -> Optional[URIRef]:
        """ Return the URI for the CURIE if a mapping is available, otherwise return None """
        try:
            return nsm.uri_for(self)
        except ValueError:
            pass
        return None


class Bool:
    """ Wrapper for boolean datatype """
    bool_true = re.compile(r'([Tt]rue)|(1)$')
    bool_false = re.compile(r'([Ff]alse)|(0)$')

    def __new__(cls, v: Union[str, bool, "Bool"]) -> bool:
        if isinstance(v, bool):
            return v
        if cls.bool_true.match(str(v)):
            return True
        if cls.bool_false.match(str(v)):
            return False
        if is_strict():
            raise ValueError(f"{v}: Must be a boolean value")
        return bool(v)

    @classmethod
    def is_valid(cls, v: str) -> bool:
        """ Determine whether the string v is a valid instance of bool """
        return isinstance(v, bool) or cls.bool_true.match(str(v)) or cls.bool_false.match(str(v))


class XSDTime(str):
    """ Wrapper for time datatype """
    def __new__(cls, value: Union[str, datetime.time, datetime.datetime, Literal]) -> str:
        if is_strict() and not cls.is_valid(value):
            raise ValueError(f"{value} is not a valid time")
        if isinstance(value, Literal):
            value = value.value
        try:
            if not isinstance(value, datetime.time):
                value = datetime.time.fromisoformat(value)
            return datetime.time.fromisoformat(str(value)).isoformat()
        except TypeError as e:
            pass
        except ValueError as e:
            pass
        if not is_strict():
            return str(value)
        raise e

    @classmethod
    def is_valid(cls, value: Union[str, datetime.time, datetime.datetime, Literal]) -> bool:
        if isinstance(value, Literal):
            value = value.value
        if isinstance(value, (datetime.time, datetime.datetime)):
            value = value.isoformat()
        try:
            datetime.time.fromisoformat(str(value))
        except ValueError:
            return False
        return True


class XSDDate(str):
    """ Wrapper for date datatype """
    def __new__(cls, value: Union[str, datetime.date, Literal]) -> str:
        if is_strict() and not cls.is_valid(value):
            raise ValueError(f"{value} is not a valid date")
        if isinstance(value, Literal):
            value = value.value
        try:
            if not isinstance(value, datetime.date):
                value = datetime.date.fromisoformat(str(value))
            return value.isoformat()
        except TypeError as e:
            pass
        except ValueError as e:
            pass
        if not is_strict():
            return str(value)
        raise e

    @classmethod
    def is_valid(cls, value: Union[str, datetime.date, Literal]) -> bool:
        if isinstance(value, Literal):
            value = value.value
        if isinstance(value, datetime.date):
            value = value.isoformat()
        try:
            datetime.date.fromisoformat(str(value))
        except ValueError:
            return False
        return True


class XSDDateTime(str):
    """ Wrapper for date time dataclass """
    def __new__(cls, value: Union[str, datetime.datetime, Literal]) -> str:
        if is_strict() and not cls.is_valid(value):
            raise ValueError(f"{value} is not a valid datetime")
        if isinstance(value, Literal):
            value = value.value
        try:
            if not isinstance(value, datetime.datetime):
                value = datetime.datetime.fromisoformat(value)      # Note that this handles non 'T' format as well
            return value.isoformat()
        except TypeError as e:
            pass
        except ValueError as e:
            pass
        if not is_strict():
            return str(value)
        raise e

    @classmethod
    def is_valid(cls, value: Union[str, datetime.datetime, Literal]) -> bool:
        if isinstance(value, Literal):
            value = value.value
        if isinstance(value, datetime.datetime):
            value = value.isoformat()
        try:
            datetime.datetime.fromisoformat(value)
        except ValueError:
            return False
        return True


class NodeIdentifier(Identifier):
    """ A RDFLib Identifier that represents a URI, CURIE or BNode in a model """

    def __new__(cls, v: Union[str, URIRef, URIorCURIE, "NodeIdentifier", BNode]) -> "NodeIdentifier":
        if hasattr(v, 'id'):        # Allows passing instances of identified objects
            v = v.id
        if isinstance(v, NodeIdentifier):
            if is_strict() and not issubclass(type(v), cls):
                raise ValueError(f"Invalid identifier type for {cls.__name__}: {v} ({type(v).__name__})")
            else:
                return super().__new__(cls, v)
        elif type(v) in (str, URIRef, URIorCURIE, BNode) or not is_strict():
            return super().__new__(cls, str(v))
        else:
            raise ValueError(f"Unknown identifier for {cls.__name__}: {v}")

    def __post_init__(self):
        pass


class ElementIdentifier(NodeIdentifier):
    """ A URIorCURIE that represents an element in a model """

    def __new__(cls, v: Union[str, URIRef, URIorCURIE, NodeIdentifier, "ElementIdentifier"]) -> "ElementIdentifier":
        if hasattr(v, 'id'):        # Allows passing instances of identified objects
            v = v.id
        if is_strict() and (type(v) is BNode or str(v).startswith('_:')):
            raise ValueError(f"Invalid identifier type for {cls.__name__}: {v} ({type(v).__name__})")
        return super().__new__(cls, v)

    def __post_init__(self):
        pass




import os
import unittest
from types import ModuleType

from jsonasobj import as_json

from biolinkml.generators.jsonldcontextgen import ContextGenerator
from biolinkml.generators.pythongen import PythonGenerator
from biolinkml.utils.yamlutils import as_rdf
from tests.test_issues import sourcedir


class Issue113TestCase(unittest.TestCase):

    def header(self, txt: str) -> str:
        return '\n' + ("=" * 20) + f" {txt} " + ("=" * 20)

    def test_issue_113(self):
        """ Make sure that types are generated as part of the output """
        yaml_fname = os.path.join(sourcedir, 'issue_113.yaml')
        python = PythonGenerator(yaml_fname).serialize()
        print(self.header("Python"))
        print(python)
        spec = compile(python, 'test', 'exec')
        module = ModuleType('test')
        exec(spec, module.__dict__)
        example = module.TestClass(test_attribute_2="foo")
        assert hasattr(example, "test_attribute_2")
        assert hasattr(example, "test_attribute_1")
        example.wiible = "foo";
        example.test_attribute_1 = "foo";
        example.test_attribute_2 = "foo";

        # JSON Representation
        print(self.header("JSON"))
        print(as_json(example))


if __name__ == '__main__':
    unittest.main()



"""
Generate CSVs
"""
import os
import sys
from csv import DictWriter
from typing import List, Union, TextIO, Set, Optional

import click

from biolinkml.meta import ClassDefinition, ClassDefinitionName, SchemaDefinition
from biolinkml.utils.formatutils import underscore, be
from biolinkml.utils.generator import Generator, shared_arguments


class CsvGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.1.1"
    valid_formats = ['csv', 'tsv']

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], **kwargs) -> None:
        super().__init__(schema, **kwargs)
        self.sep: Optional[str] = None
        self.closure: Optional[Set[ClassDefinitionName]] = None       # List of classes to include in output
        self.writer: Optional[DictWriter] = None
        
    def visit_schema(self, classes: List[ClassDefinitionName] = None, **_) -> None:
        # Note: classes comes from the "root" argument
        self.closure = set()

        if classes is None:
            classes = []

        # Validate the supplied list of classes
        for clsname in classes:
            if clsname not in self.schema.classes:
                raise ValueError(f"Unrecognized class: {clsname}")
            else:
                self.closure.update(self.ancestors(self.schema.classes[clsname]))

        dialect: str = "excel" if self.format == 'csv' else "excel-tab"
        self.writer = DictWriter(sys.stdout, ['id', 'mappings', 'description'], dialect=dialect)
        self.writer.writeheader()

    def visit_class(self, cls: ClassDefinition) -> bool:
        # TODO: find out what to do with mappings
        if not self.closure or cls.name in self.closure:
            self.writer.writerow({'id': underscore(cls.name),
                                  # 'mappings': "|".join(cls.mappings),
                                  'mappings': '',
                                  'description': be(cls.description)})
            return True
        return False


@shared_arguments(CsvGenerator)
@click.command()
@click.option("--root", "-r", default=None, multiple=True, help="Class(es) to transform")
def cli(yamlfile, root=None, **args):
    """ Generate CSV/TSV file from biolink model """
    print(CsvGenerator(yamlfile, **args).serialize(classes=root, **args))



import os
from typing import Union, TextIO, Optional

import click
from prologterms import Term, TermGenerator, PrologRenderer, SExpressionRenderer

from biolinkml.meta import SchemaDefinition, ClassDefinition, SlotDefinition
from biolinkml.utils.formatutils import camelcase, underscore
from biolinkml.utils.generator import Generator


class LogicProgramGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.0.2"
    valid_formats = ['lp', 'sexpr']
    visit_all_class_slots = True

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], fmt: Optional[str] = None) -> None:
        super().__init__(schema, fmt)
        self.P = TermGenerator() 
        self.R = PrologRenderer() if fmt == 'lp' else SExpressionRenderer()

    def visit_class(self, cls: ClassDefinition) -> bool:
        cn = underscore(cls.name)
        
        self.emit('class', cn)
        for p in cls.mixins:
            self.emit('mixin', cn, underscore(p))
        if cls.is_a:
            is_a = underscore(cls.is_a)
            self.emit('is_a', cn, is_a)
            if cls.defining_slots:
                self.emit('defining_slots', is_a, [underscore(s) for s in cls.defining_slots])
        uri = f'http://w3id.org/biolink/vocab/{camelcase(cls.name)}'

        self.emit('has_uri', cn, uri)
        return True

    def visit_slot(self, slot_name: str, slot: SlotDefinition) -> None:
        """ Add a slot definition per slot

        @param slot_name:
        @param slot:
        @return:
        """
        sn = underscore(slot_name)
        self.emit('slot', sn)
        if slot.domain:
            self.emit('domain', sn, underscore(slot.domain))
        if slot.range:
            self.emit('range', sn, underscore(slot.range))
        for p in slot.mixins:
            self.emit('mixin', sn, underscore(p))
        if slot.is_a:
            is_a = underscore(slot.is_a)

        #uri = self.owlgen._prop_uri(slot.name)
        uri = f'http://w3id.org/biolink/vocab/{sn}'
        self.emit('has_uri', sn, uri)
        if slot.multivalued:
            self.emit('multivalued', sn)
        if slot.required:
            self.emit('required', sn)
            
        
    
    def visit_class_slot(self, cls: ClassDefinition, aliased_slot_name: str, slot: SlotDefinition) -> None:
        sn = underscore(aliased_slot_name)
        cn = underscore(cls.name)
        self.emit('class_slot', cn, sn)
        self.emit('required', sn)
        self.emit('slotrange', underscore(slot.range)) if slot.range in self.schema.classes or slot.range in self.schema.types else "String"
        if slot.multivalued:
            self.emit('multivalued_in', sn, cn)
        if slot.required:
            self.emit('required_in', sn, cn)
        if slot.range:
            self.emit('range_in', sn, underscore(slot.range), cn)


    def emit(self, p, *args):
        t = Term(p, *args)
        print(f'{self.R.render(t)}.')

        

@click.command()
@click.argument("yamlfile", type=click.Path(exists=True, dir_okay=False))
@click.option("--format", "-f", default='lp', type=click.Choice(['lp', 'sexpr']), help="Output format")
def cli(yamlfile, format):
    """ Generate logic program representation of a biolink model """
    print(LogicProgramGenerator(yamlfile, format).serialize())

if __name__ == "__main__":
    cli()



import re
from contextlib import redirect_stdout
from io import StringIO
from typing import Union, Optional

from rdflib import Graph, RDF
from rdflib.compare import to_isomorphic, IsomorphicGraph, graph_diff

from biolinkml.meta import META


def to_graph(inp: Union[Graph, str], fmt: Optional[str] = "turtle") -> Graph:
    """
    Convert inp into a graph
    :param inp: Graph, file name, url or text
    :param fmt: expected format of inp
    :return: Graph representing inp
    """
    if isinstance(inp, Graph):
        return inp
    g = Graph()
    if not inp.strip().startswith('{') and '\n' not in inp and '\r' not in inp:
        with open(inp) as f:
            inp = f.read()
    g.parse(data=inp, format=fmt)
    return g


def print_triples(g: Graph) -> None:
    """
    Print the contents of g into stdout
    :param g: graph to print
    """
    g_text = re.sub(r'@prefix.*\n', '', g.serialize(format="turtle").decode())
    print(g_text)


def compare_rdf(expected: Union[Graph, str], actual: Union[Graph, str], fmt: Optional[str] = "turtle") -> Optional[str]:
    """
    Compare expected to actual, returning a string if there is a difference
    :param expected: expected RDF. Can be Graph, file name, uri or text
    :param actual: actual RDF. Can be Graph, file name, uri or text
    :param fmt: RDF format
    :return: None if they match else summary of difference
    """
    def rem_metadata(g: Graph) -> IsomorphicGraph:
        # Remove list declarations from target
        for s in g.subjects(RDF.type, RDF.List):
            g.remove((s, RDF.type, RDF.List))
        for t in g:
            if t[1] in (META.generation_date, META.source_file_date, META.source_file_size):
                g.remove(t)
        g_iso = to_isomorphic(g)
        return g_iso

    expected_graph = to_graph(expected, fmt)
    expected_isomorphic = rem_metadata(expected_graph)
    actual_graph = to_graph(actual, fmt)
    actual_isomorphic = rem_metadata(actual_graph)

    # Graph compare takes a Looong time
    in_both, in_old, in_new = graph_diff(expected_isomorphic, actual_isomorphic)
    # if old_iso != new_iso:
    #     in_both, in_old, in_new = graph_diff(old_iso, new_iso)
    old_len = len(list(in_old))
    new_len = len(list(in_new))
    if old_len or new_len:
        txt = StringIO()
        with redirect_stdout(txt):
            print("----- Missing Triples -----")
            if old_len:
                print_triples(in_old)
            print("----- Added Triples -----")
            if new_len:
                print_triples(in_new)
        return txt.getvalue()
    return None



"""Generate yuml 

https://yuml.me/diagram/scruffy/class/samples

"""
import logging
import os
from typing import Union, TextIO, Set, List, Optional, Callable, cast

import click
import requests
from rdflib import Namespace

from biolinkml.meta import ClassDefinitionName, SchemaDefinition, SlotDefinition, \
    ClassDefinition
from biolinkml.utils.formatutils import camelcase, underscore
from biolinkml.utils.generator import Generator, shared_arguments

yuml_is_a = '^-'
yuml_uses = 'uses -.->'
yuml_injected = '< -.- inject'
yuml_slot_type = ':'
yuml_inline = '++- '
yuml_inline_rev = '-++'
yuml_ref = '- '

yuml_base = 'http://yuml.me/diagram/nofunky'
yuml_scale = ''                # ';scale:180' ';scale:80' for small
yuml_dir = ';dir:TB'           # '
yuml_class = '/class/'
YUML = Namespace(yuml_base + yuml_scale + yuml_dir + yuml_class)


class YumlGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.1.1"
    valid_formats = ['yuml', 'png', 'pdf', 'jpg', 'json', 'svg']
    visit_all_class_slots = False

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], **args) -> None:
        super().__init__(schema, **args)
        self.referenced:Optional[Set[ClassDefinitionName]] = None        # List of classes that have to be emitted
        self.generated:Optional[Set[ClassDefinitionName]] = None         # List of classes that have been emitted
        self.box_generated:Optional[Set[ClassDefinitionName]] = None     # Class boxes that have been emitted
        self.associations_generated:Optional[Set[ClassDefinitionName]] = None    # Classes with associations generated
        self.focus_classes: Optional[Set[ClassDefinitionName]] = None   # Classes to be completely filled
        self.gen_classes:Optional[Set[ClassDefinitionName]] = None           # Classes to be generated
        self.output_file_name: Optional[str] = None             # Location of output file if directory used

    def visit_schema(self, classes: Set[ClassDefinitionName]=None, directory: Optional[str] = None,
                     load_image: bool=True, **_) -> None:
        if directory:
            os.makedirs(directory, exist_ok=True)
        if classes is not None:
            for cls in classes:
                if cls not in self.schema.classes:
                    raise ValueError(f"Unknown class name: {cls}")
        self.box_generated = set()
        self.associations_generated = set()
        self.focus_classes = classes
        if classes:
            self.gen_classes = self.neighborhood(list(classes)).classrefs.union(classes)
        else:
            self.gen_classes = self.synopsis.roots.classrefs
        self.referenced = self.gen_classes
        self.generated = set()
        yumlclassdef: List[str] = []
        while self.referenced.difference(self.generated):
            cn = sorted(list(self.referenced.difference(self.generated)), reverse=True)[0]
            self.generated.add(cn)
            assocs = self.class_associations(ClassDefinitionName(cn), cn in self.referenced)
            if assocs:
                yumlclassdef.append(assocs)

        yuml_url = str(YUML) + ', '.join(yumlclassdef) + \
                   (('.' + self.format) if self.format not in ('yuml', 'svg') else '')
        file_suffix = '.svg' if self.format == 'yuml' else '.' + self.format
        if directory:
            self.output_file_name = os.path.join(directory,
                                    camelcase(sorted(classes)[0] if classes else self.schema.name) + file_suffix)
            if load_image:
                resp = requests.get(yuml_url, stream=True)
                if resp.ok:
                    with open(self.output_file_name, 'wb') as f:
                        for chunk in resp.iter_content(chunk_size=2048):
                            f.write(chunk)
                else:
                    self.logger.error(f"{resp.reason} accessing {yuml_url}")
        else:
            print(str(YUML)+', '.join(yumlclassdef), end='')

    def class_box(self, cn: ClassDefinitionName) -> str:
        """ Generate a box for the class.  Populate its interior only if (a) it hasn't previously been generated and
        (b) it appears in the gen_classes list

        @param cn:
        @return:
        """
        slot_defs: List[str] = []
        if cn not in self.box_generated and (not self.focus_classes or cn in self.focus_classes):
            cls = self.schema.classes[cn]
            for slot in self.filtered_cls_slots(cn, all_slots=True, filtr=lambda s: s.range not in self.schema.classes):
                if slot.owner != slot.name:
                    mod = self.prop_modifier(cls, slot)
                    slot_defs.append(underscore(self.aliased_slot_name(slot)) +
                                     mod + ':' +
                                     underscore(slot.range) + self.cardinality(slot))
            self.box_generated.add(cn)
        self.referenced.add(cn)
        return '[' + camelcase(cn) + ('|' + ';'.join(slot_defs) if slot_defs else '') + ']'

    def class_associations(self, cn: ClassDefinitionName, must_render: bool=False) -> str:
        """ Emit all associations for a focus class.  If none are specified, all classes are generated

        @param cn: Name of class to be emitted
        @param must_render: True means render even if this is a target (class is specifically requested)
        @return: YUML representation of the association
        """

        # NOTE: YUML diagrams draw in the opposite order in which they are created, so we work from bottom to top and
        # from right to left
        assocs: List[str] = []
        if cn not in self.associations_generated and (not self.focus_classes or cn in self.focus_classes):
            cls = self.schema.classes[cn]

            # Slots that reference other classes
            for slot in self.filtered_cls_slots(cn, False, lambda s: s.range in self.schema.classes)[::-1]:
                # Swap the two boxes because, in the case of self reference, the last definition wins
                if not slot.range in self.associations_generated and slot.owner != slot.name:
                    rhs = self.class_box(cn)
                    lhs = self.class_box(cast(ClassDefinitionName, slot.range))
                    assocs.append(lhs + '<' + self.aliased_slot_name(slot) + self.prop_modifier(cls, slot) +
                                  self.cardinality(slot, False) + (yuml_inline_rev if slot.inlined else yuml_ref) + rhs)

            # Slots in other classes that reference this
            for slotname in sorted(self.synopsis.rangerefs.get(cn, [])):
                slot = self.schema.slots[slotname]
                # Don't do self references twice
                # Also, slot must be owned by the class
                if slot.owner and slot.owner != slotname and slot.domain and \
                        (slot.domain != cls.name or slot.range != cls.name) and\
                        slot.domain not in self.associations_generated:
                    dom = self.schema.classes[slot.domain]
                    assocs.append(self.class_box(slot.domain) + (yuml_inline if slot.inlined else yuml_ref) +
                                  self.aliased_slot_name(slot) + self.prop_modifier(dom, slot) +
                                  self.cardinality(slot, False) + '>' + self.class_box(cn))

            # Mixins used in the class
            for mixin in cls.mixins:
                assocs.append(self.class_box(cn) + yuml_uses + self.class_box(mixin))

            # Classes that use the class as a mixin
            if cls.name in self.synopsis.mixinrefs:
                for mixin in sorted(self.synopsis.mixinrefs[cls.name].classrefs, reverse=True):
                    assocs.append(self.class_box(ClassDefinitionName(mixin)) + yuml_uses + self.class_box(cn))

            # Classes that inject information
            if cn in self.synopsis.applytos.classrefs:
                for injector in sorted(self.synopsis.applytorefs[cn].classrefs, reverse=True):
                    assocs.append(self.class_box(cn) + yuml_injected + self.class_box(ClassDefinitionName(injector)))
            self.associations_generated.add(cn)

            # Children
            if cn in self.synopsis.isarefs:
                for is_a_cls in sorted(self.synopsis.isarefs[cn].classrefs, reverse=True):
                    assocs.append(self.class_box(cn) + yuml_is_a + self.class_box(ClassDefinitionName(is_a_cls)))

            # Parent
            if cls.is_a and cls.is_a not in self.associations_generated:
                assocs.append(self.class_box(cls.is_a) + yuml_is_a + self.class_box(cn))
        return ', '.join(assocs)

    @staticmethod
    def cardinality(slot: SlotDefinition, is_attribute: bool = True) -> str:
        if is_attribute:
            if slot.multivalued:
                return ' %2B' if slot.required else ' *'
            else:
                return '' if slot.required else ' %3F'
        else:
            if slot.multivalued:
                return ' 1..*' if slot.required else ' 0..*'
            else:
                return ' 1..1' if slot.required else ' 0..1'

    def filtered_cls_slots(self,
                           cn: ClassDefinitionName,
                           all_slots: bool=True,
                           filtr: Callable[[SlotDefinition], bool] = None) -> List[SlotDefinition]:
        """ Return the set of slots associated with the class that meet the filter criteria.  Slots will be returned
        in defining order, with class slots returned last

        @param cn: name of class to filter
        @param all_slots: True means include attributes
        @param filtr: Slot filter predicate
        @return: List of slot definitions
        """
        if filtr is None:
            filtr = lambda x: True

        rval = []
        cls = self.schema.classes[cn]
        cls_slots = self.all_slots(cls, cls_slots_first=True)
        for slot in cls_slots:
            if (all_slots or slot.range in self.schema.classes) and filtr(slot):
                rval.append(slot)

        return rval

    def prop_modifier(self, cls: ClassDefinition, slot: SlotDefinition) -> str:
        """ Return the modifiers for the slot:
            (i) - inherited
            (m) - inherited through mixin
            (a) - injected
            (pk) - primary ckey

        @param cls:
        @param slot:
        @return:
        """
        pk = '(pk)' if slot.key else ''
        inherited = slot.name not in self.own_slot_names(cls)
        mixin = inherited and slot.name in \
                [mslot.name for mslot in [self.schema.classes[m] for m in cls.mixins]]
        injected = cls.name in self.synopsis.applytos.classrefs and \
                   slot.name in [aslot.name for aslot in
                                 [self.schema.classes[a] for a in sorted(self.synopsis.applytorefs[cls.name].classrefs)]]
        return pk + ('(a)' if injected else '(m)' if mixin else '(i)' if inherited else '')


@shared_arguments(YumlGenerator)
@click.command()
@click.option("--classes", "-c", default=None, multiple=True, help="Class(es) to emit")
@click.option("--directory", "-d", help="Output directory - if supplied, YUML rendering will be saved in file")
def cli(yamlfile, **args):
    """ Generate a UML representation of a biolink model """
    print(YumlGenerator(yamlfile, **args).serialize(**args), end="")



import unittest

# This has to occur post ClickTestCase
import click

from biolinkml.generators.yumlgen import cli, YumlGenerator
from tests import source_yaml_path
from tests.test_scripts.clicktestcase import ClickTestCase


class GenYUMLTestCase(ClickTestCase):
    testdir = "genyuml"
    click_ep = cli
    prog_name = "gen-yuml"

    def test_help(self):
        self.do_test("--help", 'help')

    def test_meta(self):
        self.maxDiff = None

        self.do_test(source_yaml_path, 'meta.yuml')
        self.do_test(source_yaml_path + ' -f yuml', 'meta.yuml')
        self.do_test(source_yaml_path + ' -f xsv', 'meta_error', error=click.exceptions.BadParameter)
        self.do_test(source_yaml_path + ' -c definition', 'definition.yuml')
        self.do_test(source_yaml_path + ' -c definition -c element', 'definition_element.yuml')
        self.do_test(source_yaml_path + ' -c noclass', 'definition.yuml', error=ValueError)

        tmp_dir = self.temp_directory('meta')
        self.do_test([source_yaml_path, '-c', 'schema_definition', '-d', tmp_dir], dirbase='meta')
        tmp_dir = self.temp_directory('meta1')
        self.do_test([source_yaml_path, '-c', 'definition', '-d', tmp_dir], dirbase='meta1')
        tmp_dir = self.temp_directory('meta2')
        self.do_test([source_yaml_path, '-c', 'element', '-d', tmp_dir], dirbase='meta2')

        # Directory tests
        for fmt in YumlGenerator.valid_formats:
            if fmt != 'yuml':
                tmp_dir = self.temp_directory('meta_' + fmt)
                self.do_test([source_yaml_path, '-f', fmt, '-c', 'schema_definition', '-d', tmp_dir],
                             dirbase='meta_' + fmt)


if __name__ == '__main__':
    unittest.main()



import os
import re
import sys

import click


def compare_files(file: str, target: str, comments: str = r'^\s+#.*\n') -> int:
    def filtr(txt: str) -> str:
        return re.sub(comments, '', txt, flags=re.MULTILINE).strip()
    if os.path.exists(target):
        with open(target) as oldfile:
            oldtext = filtr(oldfile.read())
    else:
        oldtext = ""

    with open(file) as newfile:
        newtext = filtr(newfile.read())
    return int(oldtext == newtext)


@click.command()
@click.argument("file1", type=click.Path(exists=True, dir_okay=False))
@click.argument("file2", type=click.Path(dir_okay=False))
@click.option("-c", "--comments", help="Comments regexp", default="^#.*$", show_default=True)
def cli(file1, file2, comments) -> None:
    """ Compare file1 to file2 using a filter """
    sys.exit(compare_files(file1, file2, comments))


if __name__ == '__main__':
    cli(sys.argv[1:])



import os
from typing import Union, TextIO

import click

from biolinkml.meta import SchemaDefinition, ClassDefinition, SlotDefinition
from biolinkml.utils.formatutils import camelcase, lcamelcase
from biolinkml.utils.generator import Generator, shared_arguments


class GraphqlGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.1.1"
    valid_formats = ['graphql']
    visit_all_class_slots = True

    def __init__(self, schema: Union[str, TextIO, SchemaDefinition], **kwargs) -> None:
        super().__init__(schema, **kwargs)

    def visit_class(self, cls: ClassDefinition) -> bool:
        etype = 'interface' if (cls.abstract or cls.mixin) and not cls.mixins else 'type'
        mixins = ', '.join([camelcase(mixin) for mixin in cls.mixins])
        print(f"{etype} {camelcase(cls.name)}" + (f" implements {mixins}" if mixins else ""))
        print("  {")
        return True

    def end_class(self, cls: ClassDefinition) -> None:
        print("  }")
        print()

    def visit_class_slot(self, cls: ClassDefinition, aliased_slot_name: str, slot: SlotDefinition) -> None:
        slotrange = camelcase(slot.range) if slot.range in self.schema.classes or slot.range in self.schema.types else "String"
        if slot.multivalued:
            slotrange = f"[{slotrange}]"
        if slot.required:
            slotrange = slotrange + '!'
        print(f"    {lcamelcase(aliased_slot_name)}: {slotrange}")


@shared_arguments(GraphqlGenerator)
@click.command()
def cli(yamlfile, **args):
    """ Generate graphql representation of a biolink model """
    print(GraphqlGenerator(yamlfile, **args).serialize(**args))



import dataclasses
import os
from copy import deepcopy
from typing import Dict, Optional, Union, cast, List

from biolinkml.meta import SchemaDefinition, Element, SlotDefinition, ClassDefinition, TypeDefinition, \
    SlotDefinitionName, TypeDefinitionName
from biolinkml.utils.formatutils import uri_for, camelcase, underscore
from biolinkml.utils.namespaces import Namespaces


def merge_schemas(target: SchemaDefinition, mergee: SchemaDefinition, imported_from: Optional[str] = None,
                  namespaces: Optional[Namespaces] = None) -> None:
    """ Merge mergee into target """
    assert target.name is not None, "Schema name must be supplied"
    if target.license is None:
        target.license = mergee.license

    target.imports += [imp for imp in mergee.imports if imp not in target.imports]
    set_from_schema(mergee)

    if namespaces:
        merge_namespaces(target, mergee, namespaces)

    if imported_from is None:
        imported_from_uri = None
    else:
        if imported_from.startswith("http") or ":" not in imported_from:
            imported_from_uri = imported_from
        else:
            imported_from_uri = namespaces.uri_for(imported_from)
    merge_dicts(target.classes, mergee.classes, imported_from, imported_from_uri)
    merge_dicts(target.slots, mergee.slots, imported_from, imported_from_uri)
    merge_dicts(target.types, mergee.types, imported_from, imported_from_uri)
    merge_dicts(target.subsets, mergee.subsets, imported_from, imported_from_uri)


def merge_namespaces(target: SchemaDefinition, mergee: SchemaDefinition, namespaces) -> None:
    """
    Add the mergee namespace definitions to target

    :param target:
    :param mergee:
    :param namespaces:
    :return:
    """
    for prefix in mergee.prefixes.values():
        namespaces[prefix.prefix_prefix] = prefix.prefix_reference
        # if prefix.prefix_prefix not in target.prefixes:
        #     target.prefixes[prefix.prefix_prefix] = prefix
        if prefix.prefix_prefix in target.prefixes and \
                target.prefixes[prefix.prefix_prefix].prefix_reference != prefix.prefix_reference:
            raise ValueError(f'Prefix: {prefix.prefix_prefix} mismatch between {target.name} and {mergee.name}')
    for mmap in mergee.default_curi_maps:
        namespaces.add_prefixmap(mmap)


def set_from_schema(schema: SchemaDefinition) -> None:
    for t in [schema.subsets, schema.classes, schema.slots, schema.types]:
        for k in t.keys():
            t[k].from_schema = schema.id
            t[k].definition_uri = uri_for(schema.default_prefix, underscore(t[k].name
                                  if isinstance(t[k], SlotDefinition) else camelcase(t[k].name)))


def merge_dicts(target: Dict[str, Element], source: Dict[str, Element], imported_from: str, imported_from_uri: str) -> None:
    for k, v in source.items():
        if k in target and source[k].from_schema != target[k].from_schema:
            raise ValueError(f"Conflicting URIs ({source[k].from_schema}, {target[k].from_schema}) for item: {k}")
        target[k] = deepcopy(v)
        # currently all imports closures are merged into main schema, EXCEPT
        # internal biolinkml types, which are considered separate
        # https://github.com/biolink/biolinkml/issues/121
        if imported_from is not None:
            if imported_from.startswith("biolinkml") or imported_from_uri.startswith("https://w3id.org/biolink/biolinkml"):
                target[k].imported_from = imported_from


def merge_slots(target: Union[SlotDefinition, TypeDefinition], source: Union[SlotDefinition, TypeDefinition],
                skip: List[Union[SlotDefinitionName, TypeDefinitionName]] = None) -> None:
    """
    Merge slot source into target

    :param target: slot to merge into
    :param source: slot to be merged from
    :param skip: Properties to not merge (used to prevent provenance such as 'inherited from' from propagating)
    """
    if skip is None:
        skip = []
    for k, v in dataclasses.asdict(source).items():
        if k not in skip and v is not None and getattr(target, k, None) is None:
            if k in source._inherited_slots:
                setattr(target, k, deepcopy(v))
            else:
                setattr(target, k, None)


def slot_usage_name(usage_name: SlotDefinitionName, owning_class: ClassDefinition) -> SlotDefinitionName:
    """
     Synthesize a unique name for an overridden slot

    :param usage_name:
    :param owning_class:
    :return: Synthesized name
    """
    return SlotDefinitionName(owning_class.name + '_' + usage_name)


def alias_root(schema: SchemaDefinition, slotname: SlotDefinitionName) -> Optional[SlotDefinitionName]:
    """ Return the ultimate alias of a slot """
    alias = schema.slots[slotname].alias if slotname in schema.slots else None
    if alias and alias == slotname:
        raise ValueError("Error: Slot {slotname} is aliased to itself.")
    return alias_root(schema, cast(SlotDefinitionName, alias)) if alias else slotname


def merge_classes(schema: SchemaDefinition, target: ClassDefinition, source: ClassDefinition,
                  at_end: bool = False) -> None:
    """ Merge the slots in source into target

    :param schema: Containing schema
    :param target: mergee
    :param source: class to merge
    :param at_end: True means add mergee to the end.  False to the front
    """

    # List of grounded slots referenced in the target class
    target_base_slots = set(alias_root(schema, s) for s in target.slots)

    for slotname in source.slots if at_end else source.slots[::-1]:
        slotbase = alias_root(schema, slotname)
        if slotbase in target.slot_usage:
            slotname = slot_usage_name(slotbase, target)
        if slotbase not in target_base_slots:
            target.slots.append(slotname) if at_end else target.slots.insert(0, slotname)
            target_base_slots.add(slotbase)



import os
import unittest
from types import ModuleType

from biolinkml.generators.pythongen import PythonGenerator

sourcedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'source')


class Issue39UnitTest(unittest.TestCase):
    def test_python_import(self):
        """ Import generates for biolink-model """
        python = PythonGenerator(os.path.join(sourcedir, 'issue_38.yaml')).serialize()
        print(python)
        # We never get here if the imports fails
        self.assertEqual(True, True)
        spec = compile(python, 'test', 'exec')
        module = ModuleType('test')
        exec(spec, module.__dict__)


if __name__ == '__main__':
    unittest.main()



import os
import re
import unittest
# This has to occur post ClickTestCase
from functools import reduce
from typing import List, Tuple
from urllib.parse import urljoin

import click
from rdflib import Graph, URIRef
from rdflib.collection import Collection

from biolinkml import METAMODEL_NAMESPACE
from biolinkml.generators.jsonldcontextgen import ContextGenerator
from biolinkml.generators.jsonldgen import cli, JSONLDGenerator
from tests import source_yaml_path
from tests.test_scripts import testscriptstempdir
from tests.test_scripts.clicktestcase import ClickTestCase
from tests.utils.generator_utils import BIOLINK_IMPORT_MAP

cwd = os.path.dirname(__file__)
meta_context = 'file:./output/gencontext/meta.jsonld'

repl: List[Tuple[str, str]] = [
    (r'"source_file_size": [0-9]+', ''),
    (r'"source_file_date": "[^"]+"', ''),
    (r'"generation_date": "[^"]+"', ''),
    (r'"source_file": "[^"]+"', '')
]


def filtr(txt: str) -> str:
    return reduce(lambda s, expr: re.sub(expr[0], expr[1], s), repl, txt)


class GenJSONLDTestCase(ClickTestCase):
    testdir = "genjsonld"
    click_ep = cli
    prog_name = "gen-jsonld"

    def test_help(self):
        self.do_test("--help", 'help')

    def test_meta(self):
        self.maxDiff = None
        self.do_test(source_yaml_path + f" --context {meta_context}", 'meta.jsonld', filtr=filtr)
        self.do_test(source_yaml_path + f' -f jsonld --context {meta_context}', 'meta.jsonld', filtr=filtr)
        self.do_test(source_yaml_path + f' -f xsv --context {meta_context}', 'meta_error',
                     error=click.exceptions.BadParameter)

    def check_size(self, g: Graph, g2: Graph, root: URIRef, expected_classes: int,
                   expected_slots: int, expected_types: int, model: str) -> None:
        for graph in [g, g2]:

            n_classes = len(list(graph.objects(root, METAMODEL_NAMESPACE.classes)))
            n_slots = len(list(graph.objects(root, METAMODEL_NAMESPACE.slots)))
            n_types = len(list(graph.objects(root, METAMODEL_NAMESPACE.types)))
            self.assertEqual(expected_classes, n_classes, f"Expected {expected_classes} classes in {model}")
            self.assertEqual(expected_slots, n_slots, f"Expected {expected_slots} slots in {model}")
            self.assertEqual(expected_types, n_types, f"Expected {expected_types} types in {model}")

    def test_meta_output(self):
        """ Generate a context AND a jsonld for the metamodel and make sure it parses as RDF """
        jsonld_path = os.path.join(testscriptstempdir, 'metajson.jsonld')
        rdf_path = os.path.join(testscriptstempdir, 'metardf.ttl')
        meta_context_path = os.path.join(testscriptstempdir, 'metacontext.jsonld')

        # Generate an image of the metamodel
        gen = ContextGenerator(source_yaml_path, importmap=BIOLINK_IMPORT_MAP)
        base = gen.schema.id
        if base[-1] not in '/#':
            base += '/'
        base += gen.schema.name
        with open(meta_context_path, 'w') as tfile:
            tfile.write(gen.serialize())
        with open(jsonld_path, 'w') as tfile:
            tfile.write(JSONLDGenerator(source_yaml_path, fmt=JSONLDGenerator.valid_formats[0],
                                        importmap=BIOLINK_IMPORT_MAP).serialize(context=meta_context_path))
        g = Graph()
        g.load(jsonld_path, format="json-ld")
        g.serialize(rdf_path, format="ttl")
        g.bind('meta', METAMODEL_NAMESPACE)
        new_ttl = g.serialize(format="turtle").decode()
        new_g = Graph()
        new_g.parse(data=new_ttl, format="turtle")
        self.check_size(g, new_g, URIRef(base), 11, 91, 13, "meta")


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from rdflib import Namespace, Graph, XSD

from biolinkml import LOCAL_TYPES_YAML_FILE
from biolinkml.generators.rdfgen import RDFGenerator

META = Namespace("https://w3id.org/biolink/biolinkml/meta/")
METATYPE = Namespace("https://w3id.org/biolink/biolinkml/type/")


class DateTestCase(unittest.TestCase):
    def test_date_time(self):
        """ date datatype should be rdf:date and datetime rdf:datetime """
        rdf = RDFGenerator(LOCAL_TYPES_YAML_FILE).serialize()
        g = Graph()
        g.parse(data=rdf, format="turtle")
        self.assertEqual(XSD.date, g.value(METATYPE.date, META.uri))
        self.assertEqual(XSD.dateTime, g.value(METATYPE.datetime, META.uri))


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from biolinkml.generators.pythongen import PythonGenerator
from tests.test_issues import sourcedir


class SlotSubclassTestCase(unittest.TestCase):
    def test_slot_subclass_good(self):
        """ Test slot domain as superclass of parent """

        yaml_fname = os.path.join(sourcedir, 'issue_56_good.yaml')
        PythonGenerator(yaml_fname).serialize()
        self.assertTrue(True, "Didn't throw an error - we good")
        yaml_fname = os.path.join(sourcedir, 'issue_56_bad.yaml')
        PythonGenerator(yaml_fname).serialize()

    @unittest.skipIf(True, "We still need to figure out what to do here")
    def test_slot_subclass_bad(self):
        """ Test slot domain as superclass of parent """

        yaml_fname = os.path.join(sourcedir, 'issue_56_bad.yaml')
        with self.assertRaises(ValueError):
            PythonGenerator(yaml_fname).serialize()


if __name__ == '__main__':
    unittest.main()



from dataclasses import dataclass, field
from typing import Dict, Set, List, Union, cast
import logging

from biolinkml.meta import SchemaDefinition, Element, Definition, ClassDefinition, SlotDefinitionName, \
    ClassDefinitionName, TypeDefinitionName, DefinitionName, SlotDefinition, ElementName, TypeDefinition, \
    SubsetDefinitionName
from biolinkml.utils.metamodelcore import empty_dict
from biolinkml.utils.typereferences import RefType, ClassType, TypeType, SlotType, References, SubsetType


def empty_references() -> field:
    return field(default_factory=References)


ClassOrSlotName = Union[ClassDefinitionName, TypeDefinitionName]


@dataclass
class SchemaSynopsis:
    schema: SchemaDefinition = field(repr=False, compare=False)

    # References by type -- set by add_ref
    typerefs: Dict[TypeDefinitionName, References] = empty_dict()       # Type name to all references
    slotrefs: Dict[SlotDefinitionName, References] = empty_dict()       # Slot name to all references
    classrefs: Dict[ClassDefinitionName, References] = empty_dict()     # Class name to all references
    subsetrefs: Dict[SubsetDefinitionName, References] = empty_dict()            # Subset name to references

    # Type specific
    typebases: Dict[str, Set[TypeDefinitionName]] = empty_dict()          # Base referencing types (direct and indirect)
    typeofs: Dict[TypeDefinitionName, TypeDefinitionName] = empty_dict()  # Type to specializations

    # Slot specific
    slotclasses: Dict[SlotDefinitionName, Set[ClassDefinitionName]] = empty_dict()    # Slot to including classes
    definingslots: Dict[SlotDefinitionName, Set[ClassDefinitionName]] = empty_dict()  # Slot to defining decls
    slotusages: Dict[SlotDefinitionName, Set[ClassDefinitionName]] = empty_dict()     # Slot to overriding classes
    owners: Dict[SlotDefinitionName, Set[ClassDefinitionName]] = empty_dict()         # Slot to owning classes (sb. 1)
    inverses: Dict[str, Set[str]] = empty_dict()                        # Slots declared as inverses of other slots

    # Class specific
    ownslots: Dict[ClassDefinitionName, Set[SlotDefinitionName]] = empty_dict() # Slots directly owned by class

    # Class to slot domains == class.slots

    # Slot or Class (Definition) specific
    roots: References = empty_references()                      # Definitions with no parents
    isarefs: Dict[DefinitionName, References] = empty_dict()    # Definition to isa references
    mixinrefs: Dict[DefinitionName, References] = empty_dict()  # Mixin to referencing classes or slots
    mixins: References = empty_references()                     # Definitions declared as mixin
    abstracts: References = empty_references()                  # Definitions declared as abstract
    applytos: References = empty_references()                   # Definitions that include applytos
    applytorefs: Dict[DefinitionName, References] = empty_dict()  # Definition to applyier

    # Slot or Type specific
    rangerefs: Dict[ElementName, Set[SlotDefinitionName]] = empty_dict()  # Type or class to range slot

    # Element - any type
    inschema: Dict[str, References] = empty_references()        # Schema name to elements

    def __post_init__(self):
        for k, v in self.schema.slots.items():
            self.summarize_slot_definition(k, v)
        for k, v in self.schema.types.items():
            self.summarize_type_definition(k, v)
        for k, v in self.schema.classes.items():
            self.summarize_class_definition(k, v)

        # Generate a list of slots owned exclusively by cls
        for cls in self.schema.classes.values():
            non_owned_slots = set()
            if cls.is_a:
                non_owned_slots = set(self.schema.classes[cls.is_a].slots)
            for mixin in cls.mixins:
                non_owned_slots.update(set(self.schema.classes[mixin].slots))
            owned_slots = set(cls.slots) - non_owned_slots
            self.ownslots[cls.name] = set(cls.slots) - non_owned_slots
            for slotname in owned_slots:
                self.owners.setdefault(slotname, set()).add(cls.name)


    def summarize_slot_definition(self, k: SlotDefinitionName, v: SlotDefinition) -> None:
        """
        Summarize a slot definition
        :param k: slot name
        :param v: slot definition
        :return:
        """
        self.summarize_definition(SlotType, k, v)
        if v.domain:
            self.add_ref(SlotType, k, ClassType, v.domain)
        self.rangerefs.setdefault(v.range, set()).add(k)
        self.add_ref(SlotType, k, ClassType if v.range in self.schema.classes else TypeType, v.range)

    def summarize_type_definition(self, k: TypeDefinitionName, v: TypeDefinition):
        """
        Summarize type definition

        :param k: Type name
        :param v: Type definition
        :return:
        """
        self.summarize_element(TypeType, k, v)
        if v.typeof:
            self.typeofs.setdefault(v.typeof, set()).add(k)
            self.add_ref(TypeType, k, TypeType, v.typeof)
        if v.base:
            self.typebases.setdefault(v.base, set()).add(k)

    def summarize_class_definition(self, k: ClassDefinitionName, v: ClassDefinition) -> None:
        """
        Summarize class definition element

        :param k: Class name
        :param v: Class definition
        :return:
        """
        self.summarize_definition(ClassType, k, v)
        for slotname in v.slots:
            self.add_ref(ClassType, k, SlotType, slotname)
        for slotname, usage in v.slot_usage.items():
            self.slotusages.setdefault(slotname, set()).add(k)
            self.add_ref(ClassType, k, SlotType, slotname)
            slot_alias = self.schema.slots[slotname].alias
            if slot_alias:
                self.add_ref(SlotType, slotname, SlotType, cast(SlotDefinitionName, slot_alias))
                self.add_ref(ClassType, k, SlotType, cast(SlotDefinitionName, slot_alias))

    def summarize_definition(self, typ: RefType, k: DefinitionName, v: Definition) -> None:
        """
        Summarize slot and class definitions

        :param typ: type (slot or class)
        :param k: name
        :param v: definition
        :return:
        """
        self.summarize_element(typ, k, v)
        if v.is_a:
            self.isarefs.setdefault(v.is_a, References()).addref(typ, k)
            self.add_ref(typ, k, typ, v.is_a)
        else:
            self.roots.addref(typ, k)
        if v.abstract:
            self.abstracts.addref(typ, k)
        if v.mixin:
            self.mixins.addref(typ, k)
        for mixin in v.mixins:
            self.mixinrefs.setdefault(mixin, References()).addref(typ, k)
            self.add_ref(typ, k, typ, mixin)
        if v.apply_to:
            self.applytos.addref(typ, k)
        for applyto in v.apply_to:
            self.applytorefs.setdefault(applyto, References()).addref(typ, k)
            self.add_ref(typ, k, typ, applyto)

    def summarize_element(self, typ: RefType, k: ElementName, v: Element) -> None:
        """
         Summarize element level items

        :param typ: element type
        :param k: element name
        :param v: element deffinition
        :return:
        """
        if k != v.name:
            raise ValueError("{typ} name mismatch: {k} != {v.name}")        # should never happen
        for subset in v.in_subset:
            self.add_ref(typ, k, SubsetType, subset)

    def add_ref(self, fromtype: RefType, fromname: ElementName, totype: RefType, toname: ElementName, ) -> None:
        """ Add an inverse reference, indicating that to type/name is referenced by from type/name

        :param fromtype: Referencer type
        :param fromname: Referencer name
        :param totype: Referencee type
        :param toname: Referencee name
        :return:
        """
        if totype is ClassType:
            self.classrefs.setdefault(cast(ClassDefinitionName, toname), References()).addref(fromtype, fromname)
        elif totype is SlotType:
            self.slotrefs.setdefault(cast(SlotDefinitionName, toname), References()).addref(fromtype, fromname)
        elif totype is TypeType:
            self.typerefs.setdefault(cast(TypeDefinitionName, toname), References()).addref(fromtype, fromname)
        elif totype is SubsetType:
            self.subsetrefs.setdefault(cast(SubsetDefinitionName, toname), References()).addref(fromtype, fromname)
        else:
            raise TypeError("Unknown typ: {typ}")

    def _ancestor_is_owned(self, slot: SlotDefinition) -> bool:
        return bool(slot.is_a) and (slot.is_a in self.owners or self._ancestor_is_owned(self.schema.slots[slot.is_a]))

    def errors(self) -> List[str]:

        rval = []
        undefined_classes = set(self.classrefs.keys()) - set(self.schema.classes.keys())
        if undefined_classes:
            rval += [f"\tUndefined class references: {', '.join(undefined_classes)}"]
        undefined_slots = set(self.slotrefs.keys()) - set(self.schema.slots.keys())
        if undefined_slots:
            rval += [f"\tUndefined slot references: {', '.join(undefined_slots)}"]
        undefined_types = set(self.typerefs.keys()) - set(self.schema.types.keys())
        if undefined_types:
            rval += [f"\tUndefined type references: {', '.join(undefined_types)}"]
        undefined_subsets = set(self.subsetrefs.keys()) - set(self.schema.subsets.keys())
        if undefined_subsets:
            rval += [f"\tUndefined subset references: {', '.join(undefined_subsets)}"]

        # Look for slot domain / class slot mismatches
        for slotname, owners in self.owners.items():
            owners = list(owners)
            # if len(owners) > 1:
            #     owners_str = ', '.join(owners)
            #     rval += [f"\tSlot {slotname} has multiple owners: ({owners_str})"]
            real_owner = self.schema.slots[slotname].owner
            if real_owner is None or (real_owner != slotname and real_owner != owners[0]):
                msg = f'\tSlot "{slotname}" owner ({self.schema.slots[slotname].owner}) does not match {owners[0]}'
                logging.warning(msg)
                # TODO: discuss this with Harold. not sure this merits a warning
                #rval += [msg]
        for slotname, slot in sorted(self.schema.slots.items(), key=lambda e: e[0]):
            if slotname not in self.owners:
                # Lack of ownership is no longer a sin
                # if not self._ancestor_is_owned(slot):
                #     rval += [f"\tSlot {slotname} has no owners"]
                pass
            else:
                owner = self.owners[slotname]
                # if slot.domain and (slot.domain not in self.ownslots or slotname not in self.ownslots[slot.domain]):
                #     rval += [f'\tDomain mismatch: slot "{slotname}" domain is: '
                #             f'"{slot.domain}" class "{owner}" claims ownership']

        # Inlined slots must be multivalued (not a inviolable rule, but we make assumptions about this elsewhere in
        # the python generator
        for slot in self.schema.slots.values():
            if slot.inlined and not slot.multivalued and slot.identifier:
                rval += [f'\tSlot {slot.name} is declared inline but single valued']
        return rval

    def summary(self) -> str:

        def summarize_refs(refs: Dict[ElementName, References]) -> str:
            clsrefs, slotrefs, typerefs = set(), set(), set()
            if refs is not None:
                for cr in refs.values():
                    clsrefs.update(cr.classrefs)
                    slotrefs.update(cr.slotrefs)
                    typerefs.update(cr.typerefs)
            return f"\tReferenced by: {len(clsrefs)} classes, {len(slotrefs)} slots, {len(typerefs)} types "

        def recurse_types(typ: TypeDefinitionName, indent: str='\t\t\t') -> List[str]:
            rval = [f"{indent}{typ}" + (':' if typ in self.typeofs else '')]
            if typ in sorted(self.typeofs):
                for tr in sorted(self.typeofs[typ]):
                    rval += recurse_types(tr, indent + '\t')
            return rval

        rval = ['']
        rval += [f"Classes: {len(self.schema.classes.keys())}"]
        rval += [summarize_refs(self.classrefs)]

        rval += [f"\tRoot: {len(self.roots.classrefs)}"]
        leaves = set(self.classrefs.keys()) - set(self.isarefs.keys())
        rval += [f"\tLeaf: {len(leaves)}"]
        # Standalone ar classes that are both roots and leaves
        rval += [f"\tStandalone: {len(set(self.roots.classrefs).union(set(leaves)))}"]
        rval += [f"\tDeclared mixin: {len(self.mixins.classrefs)}"]
        undeclared_mixins = set(self.mixinrefs.keys())\
            .intersection(set(self.schema.classes.keys()) - set(self.mixins.classrefs))
        rval += [f"\tUndeclared mixin: {len(undeclared_mixins)}"]
        if undeclared_mixins:
            for udm in sorted(undeclared_mixins):
                rval += [f"\t\t{udm}"]
        rval += [f"\tAbstract: {len(self.abstracts.classrefs)}"]
        undefined_classes = set(self.classrefs.keys()) - set(self.schema.classes.keys())
        if undefined_classes:
            rval += [f"\tUndefined references: {', '.join(undefined_classes)}"]

        rval += ['']
        rval += [f"Slots: {len(self.schema.slots.keys())}"]
        rval += [summarize_refs(self.slotrefs)]
        rval += [f"\tRoot: {len(self.roots.slotrefs)}"]
        leaves = set(self.slotrefs.keys()) - set(self.isarefs.keys())
        rval += [f"\tLeaf: {len(leaves)}"]
        rval += [f"\tStandalone: {len(set(self.roots.classrefs).union(set(leaves)))}"]
        rval += [f"\tDeclared mixin: {len(self.mixins.slotrefs)}"]
        undeclared_mixins = set(self.mixinrefs.keys()) \
            .intersection(set(self.schema.slots.keys()) - set(self.mixins.slotrefs))
        rval += [f"\tUndeclared mixin: {len(undeclared_mixins)}"]
        if undeclared_mixins:
            for udm in sorted(undeclared_mixins):
                rval += [f"\t\t{udm}"]
        rval += [f"\tAbstract: {len(self.abstracts.slotrefs)}"]

        # Slots that are referenced but not defined
        undefined_slots = set(self.slotrefs.keys()) - set(self.schema.slots.keys())
        if undefined_slots:
            rval += [f"\tUndefined: {len(undefined_slots)}"]

        # Slots that are defined but do not (directly) occur in any class
        n_unreferenced_descendants: int = 0
        unowned_slots: Set[SlotDefinitionName] = set()
        for slotname, slot in sorted(self.schema.slots.items(), key=lambda e: e[0]):
            if slotname not in self.owners:
                if slot.domain:
                    if self._ancestor_is_owned(slot):
                        n_unreferenced_descendants += 1
                    else:
                        unowned_slots.add(slotname)
        if n_unreferenced_descendants:
            rval += [f"\tUnreferenced descendants of owned slots: {n_unreferenced_descendants}"]
        if unowned_slots:
            rval += [f"\t* Unowned slots: {', '.join(sorted(unowned_slots))}"]


        not_in_domain: Set[SlotDefinitionName] = set()
        domain_mismatches: Set[SlotDefinitionName] = set()
        unkdomains: Set[SlotDefinitionName] = set()
        emptydomains: Set[SlotDefinitionName] = set()

        for slot in self.schema.slots.values():
            if not slot.domain:
                emptydomains.add(slot.name)
            elif slot.domain in self.schema.classes:
                if slot.name in self.schema.classes[slot.domain].slots:
                    pass
                elif slot.name not in self.slotclasses:
                    not_in_domain.add(slot.name)
                else:
                    domain_mismatches.add(slot.name)
            else:
                unkdomains.add(f"{slot.name}: {slot.domain}")
        if not_in_domain:
            rval += [f"\tNot in domain: {len(not_in_domain)}"]
            rval += ["\t\tslot.name: slot.domain"]
            rval += ["\t\t---------  -----------"]
            for slotname in sorted(not_in_domain):
                rval.append(f'\t\t"{slotname}": "{self.schema.slots[slotname].domain}"')
        if domain_mismatches:
            rval += [f"\t\tMismatches: {len(domain_mismatches)}"]
            for slotname in sorted(domain_mismatches):
                rval.append(f'\t\t\tSlot: "{slotname}" declared domain: "{self.schema.slots[slotname].domain}" '
                            f'actual domain(s): {", ".join(self.slotclasses[slotname])}')
        if unkdomains:
            rval += [f"\t* Unknown domain: {', '.join(sorted(unkdomains))}"]
        if emptydomains:
            rval += [f"\tDomain unspecified: {len(emptydomains)}"]

        rval += ["\tRanges:"]
        rval += ["\t\tType:"]
        for rng, slots in sorted(self.rangerefs.items()):
            if rng in self.schema.types:
                rval += [f"\t\t\t{rng}: {len(slots)}"]
        rval += ["\t\tClass:"]
        for rng, slots in sorted(self.rangerefs.items()):
            if rng in self.schema.classes:
                rval += [f"\t\t\t{rng}: {len(slots)}"]
        unknowns = []
        for rng, slots in sorted(self.rangerefs.items()):
            if rng not in self.schema.types and rng not in self.schema.classes:
                unknowns += [f"\t\t\t{rng}: {len(slots)}"]
        if unknowns:
            rval += ["\t\tUnknown:"] + unknowns

        shared_class_slots = set(self.schema.classes.keys()).intersection(set(self.schema.slots.keys()))
        if shared_class_slots:
            rval += ["\nClasses and Slots with the same name:"]
            for ssc in sorted(shared_class_slots):
                rval += [f"\t{ssc}"]

        rval += ['']
        rval += [f"Types: {len(self.schema.types)}"]
        rval += [summarize_refs(self.typerefs)]
        rval += ["\tBases:"]
        for base in sorted(self.typebases.keys()):
            rval += [f"\t\t{base}:"]
            for typ in sorted(self.typebases[base]):
                if not self.schema.types[typ].typeof:
                    rval += recurse_types(typ)

        return '\n'.join(rval)




import os
import unittest
import json
from types import ModuleType

from biolinkml.generators.pythongen import PythonGenerator
from tests.test_issues import sourcedir


class IssuePythonOrderingTestCase(unittest.TestCase):

    def header(self, txt: str) -> str:
        return '\n' + ("=" * 20) + f" {txt} " + ("=" * 20)

    def test_issue_python_ordering(self):
        """ Make sure that types are generated as part of the output """
        yaml_fname = os.path.join(sourcedir, 'issue_134.yaml')
        gen = PythonGenerator(yaml_fname)
        py = gen.serialize()
        print(self.header("Py"))
        print(py)
        spec = compile(py, 'test', 'exec')
        module = ModuleType('test')
        exec(spec, module.__dict__)
        e = module.E('id:1')
        b = module.B('id:2')
        e.has_b = b


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from biolinkml.utils.schemaloader import SchemaLoader
from tests.test_utils import inputdir


class URLImportTestCase(unittest.TestCase):

    def test_import_from_url(self):
        """ Validate default slot range settings """
        schema = SchemaLoader(os.path.join(inputdir, 'inherited_slots.yaml')).resolve()
        self.assertTrue('same as' in schema.classes['named thing'].slots)


if __name__ == '__main__':
    unittest.main()



# Auto generated from organization.yaml by pythongen.py version: 0.4.0
# Generation date: 2020-06-07 14:59
# Schema: organization
#
# id: http://example.org/sample/organization
# description:
# license:

import dataclasses
import sys
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from biolinkml.utils.slot import Slot
from biolinkml.utils.metamodelcore import empty_list, empty_dict, bnode
from biolinkml.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
if sys.version_info < (3, 7, 6):
    from biolinkml.utils.dataclass_extensions_375 import dataclasses_init_fn_with_kwargs
else:
    from biolinkml.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from biolinkml.utils.formatutils import camelcase, underscore, sfx
from rdflib import Namespace, URIRef
from biolinkml.utils.curienamespace import CurieNamespace


metamodel_version = "1.4.3"

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
XSD = CurieNamespace('xsd', 'http://example.org/UNKNOWN/xsd/')
DEFAULT_ = CurieNamespace('', 'http://example.org/sample/organization/')


# Types
class YearCount(int):
    type_class_uri = XSD.int
    type_class_curie = "xsd:int"
    type_name = "yearCount"
    type_model_uri = URIRef("http://example.org/sample/organization/YearCount")


class String(str):
    type_class_uri = XSD.string
    type_class_curie = "xsd:string"
    type_name = "string"
    type_model_uri = URIRef("http://example.org/sample/organization/String")


# Class references
class OrganizationId(extended_str):
    pass


class EmployeeId(extended_str):
    pass


class ManagerId(EmployeeId):
    pass


@dataclass
class Organization(YAMLRoot):
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = URIRef("http://example.org/sample/organization/Organization")
    class_class_curie: ClassVar[str] = None
    class_name: ClassVar[str] = "organization"
    class_model_uri: ClassVar[URIRef] = URIRef("http://example.org/sample/organization/Organization")

    id: Union[str, OrganizationId]
    name: Optional[str] = None
    has_boss: Optional[Union[dict, "Manager"]] = None

    def __post_init__(self, **kwargs: Dict[str, Any]):
        if self.id is None:
            raise ValueError(f"id must be supplied")
        if not isinstance(self.id, OrganizationId):
            self.id = OrganizationId(self.id)
        if self.has_boss is not None and not isinstance(self.has_boss, Manager):
            self.has_boss = Manager(self.has_boss)
        super().__post_init__(**kwargs)


@dataclass
class Employee(YAMLRoot):
    """
    A person
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = URIRef("http://example.org/sample/organization/Employee")
    class_class_curie: ClassVar[str] = None
    class_name: ClassVar[str] = "employee"
    class_model_uri: ClassVar[URIRef] = URIRef("http://example.org/sample/organization/Employee")

    id: Union[str, EmployeeId]
    last_name: str
    first_name: Optional[str] = None
    aliases: List[str] = empty_list()
    age_in_years: Optional[int] = None

    def __post_init__(self, **kwargs: Dict[str, Any]):
        if self.id is None:
            raise ValueError(f"id must be supplied")
        if not isinstance(self.id, EmployeeId):
            self.id = EmployeeId(self.id)
        if self.last_name is None:
            raise ValueError(f"last_name must be supplied")
        super().__post_init__(**kwargs)


@dataclass
class Manager(Employee):
    """
    An employee who manages others
    """
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = URIRef("http://example.org/sample/organization/Manager")
    class_class_curie: ClassVar[str] = None
    class_name: ClassVar[str] = "manager"
    class_model_uri: ClassVar[URIRef] = URIRef("http://example.org/sample/organization/Manager")

    id: Union[str, ManagerId] = None
    last_name: str = None
    has_employees: Dict[Union[str, EmployeeId], Union[dict, Employee]] = empty_dict()

    def __post_init__(self, **kwargs: Dict[str, Any]):
        if self.id is None:
            raise ValueError(f"id must be supplied")
        if not isinstance(self.id, ManagerId):
            self.id = ManagerId(self.id)
        for k, v in self.has_employees.items():
            if not isinstance(v, Employee):
                self.has_employees[k] = Employee(id=k, **({} if v is None else v))
        super().__post_init__(**kwargs)



# Slots
class slots:
    pass

slots.id = Slot(uri=DEFAULT_.id, name="id", curie=DEFAULT_.curie('id'),
                      model_uri=DEFAULT_.id, domain=None, range=URIRef)

slots.name = Slot(uri=DEFAULT_.name, name="name", curie=DEFAULT_.curie('name'),
                      model_uri=DEFAULT_.name, domain=None, range=Optional[str])

slots.aliases = Slot(uri=DEFAULT_.aliases, name="aliases", curie=DEFAULT_.curie('aliases'),
                      model_uri=DEFAULT_.aliases, domain=None, range=List[str])

slots.first_name = Slot(uri=DEFAULT_.first_name, name="first name", curie=DEFAULT_.curie('first_name'),
                      model_uri=DEFAULT_.first_name, domain=None, range=Optional[str])

slots.last_name = Slot(uri=DEFAULT_.last_name, name="last name", curie=DEFAULT_.curie('last_name'),
                      model_uri=DEFAULT_.last_name, domain=None, range=Optional[str])

slots.age_in_years = Slot(uri=DEFAULT_.age_in_years, name="age in years", curie=DEFAULT_.curie('age_in_years'),
                      model_uri=DEFAULT_.age_in_years, domain=None, range=Optional[int])

slots.has_employees = Slot(uri=DEFAULT_.has_employees, name="has employees", curie=DEFAULT_.curie('has_employees'),
                      model_uri=DEFAULT_.has_employees, domain=None, range=Dict[Union[str, EmployeeId], Union[dict, Employee]])

slots.has_boss = Slot(uri=DEFAULT_.has_boss, name="has boss", curie=DEFAULT_.curie('has_boss'),
                      model_uri=DEFAULT_.has_boss, domain=None, range=Optional[Union[dict, Manager]])

slots.employee_last_name = Slot(uri=DEFAULT_.last_name, name="employee_last name", curie=DEFAULT_.curie('last_name'),
                      model_uri=DEFAULT_.employee_last_name, domain=Employee, range=str)



import os
import unittest

from biolinkml import LOCAL_METAMODEL_LDCONTEXT_FILE
from biolinkml.generators.rdfgen import RDFGenerator
from tests.test_issues import sourcedir

@unittest.skipIf(True, "Test is not complete -- keep until fully resolved")
class NamespaceIssueTestCase(unittest.TestCase):
    def test_namespace(self):
        yaml_fname = os.path.join(sourcedir, 'issue_namespace.yaml')
        context = "https://biolink.github.io/biolink-model/context.jsonld"
        print(RDFGenerator(yaml_fname).serialize(context=context))

if __name__ == '__main__':
    unittest.main()



import os
import unittest
import json

from biolinkml.generators.jsonschemagen import JsonSchemaGenerator
from biolinkml.generators.owlgen import OwlSchemaGenerator
from tests.test_issues import sourcedir


class IssueJSONSchemaTypesTestCase(unittest.TestCase):

    def header(self, txt: str) -> str:
        return '\n' + ("=" * 20) + f" {txt} " + ("=" * 20)

    def test_issue_types(self):
        """ Make sure that types are generated as part of the output """
        yaml_fname = os.path.join(sourcedir, 'issue_129.yaml')
        gen = JsonSchemaGenerator(yaml_fname)
        gen.topCls = 'c'
        jsonschema = gen.serialize()
        print(self.header("JSONSchema"))
        print(jsonschema)
        sobj = json.loads(jsonschema)
        defs = sobj['definitions']
        C = defs['C']
        props = C['properties']
        assert props['age_in_years']['type'] == 'integer'
        assert props['has_prop']['type'] == 'boolean'
        # multivalued primitive type, inlined
        assert props['scores']['type'] == 'array'
        assert props['scores']['items']['type'] == 'number'
        # single-valued complex type, inlined
        assert props['has_d']['$ref'] == "#/definitions/D"

        # multi-valued, inlined
        assert props['has_ds']['type'] == 'array'
        assert props['has_ds']['items']['$ref'] == "#/definitions/D"

        # single-valued, non-inlined (foreign key)
        assert props['parent']['type'] == "string"

        # multi-valued, non-inlined (foreign key)
        assert props['children']['type'] == 'array'
        assert props['children']['items']['type'] == "string"

if __name__ == '__main__':
    unittest.main()



import os
import unittest
from types import ModuleType

from biolinkml.generators.pythongen import PythonGenerator
from tests.utils.metadata_filters import metadata_filter
from tests.test_utils import inputdir, outputdir
from tests.utils.generator_utils import GeneratorTestCase


class IfAbsentTestCase(GeneratorTestCase):
    root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))

    source_path: str = inputdir
    target_path: str = outputdir
    model_path: str = source_path
    model_name: str = "ifabsents"
    output_name: str = None

    def do_test(self):
        """ Test the metadata options"""
        self.single_file_generator('py', PythonGenerator, filtr=metadata_filter)

        # Make sure the python is valid
        with open(os.path.join(self.source_path, f'{IfAbsentTestCase.model_name}.py')) as f:
            pydata = f.read()
        spec = compile(pydata, 'test', 'exec')
        module = ModuleType('test')
        exec(spec, module.__dict__)

    def test_good_ifabsent(self):
        """ Test isabsent with no default_prefix """
        IfAbsentTestCase.model_name = "ifabsents"
        self.do_test()

    def test_good_ifabsent2(self):
        """ Test isabsents with default_prefix specified """
        IfAbsentTestCase.model_name = "ifabsents2"
        self.do_test()

    def test_good_ifabsent3(self):
        """ Test isabsent with no default_prefix, but prefix specified that matches the module id """
        IfAbsentTestCase.model_name = "ifabsents3"
        self.do_test()

    def test_bad_ifabsent(self):
        IfAbsentTestCase.model_name = "ifabsents_error"
        with self.assertRaises(ValueError):
            self.single_file_generator('py', PythonGenerator, filtr=metadata_filter)

    def test_ifabsent_uri(self):
        IfAbsentTestCase.model_name = "ifabsent_uri"
        self.do_test()


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from biolinkml.generators.golrgen import GolrSchemaGenerator
from tests import targetdir
from tests.test_utils import inputdir
from tests.utils.dirutils import make_and_clear_directory


class OrphanSlotUsageTestCase(unittest.TestCase):

    def test_orphan_slot_usage(self):
        """ Make sure an orphan slot_usage works """
        # The bug is that this goes into an endless loop
        output_dir = os.path.join(targetdir, 'slottest')
        make_and_clear_directory(output_dir)
        GolrSchemaGenerator(os.path.join(inputdir, 'orphan_slot_usage.yaml')).serialize(directory=output_dir)
        make_and_clear_directory(output_dir)
        self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()



import json
import os
from io import TextIOWrapper
from typing import Optional, Union, List, Any, Dict

import yaml
from jsonasobj import JsonObj, loads

CONTEXT_TYPE = Union[str, dict, JsonObj]
CONTEXTS_PARAM_TYPE = Optional[Union[CONTEXT_TYPE, List[CONTEXT_TYPE]]]


def merge_contexts(contexts: CONTEXTS_PARAM_TYPE = None, base: Optional[Any] = None,
                   root_directory: Optional[str] = None) -> JsonObj:
    """ Take a list of JSON-LD contexts, which can be one of:
        * the name of a JSON-LD file
        * the URI of a JSON-lD file
        * JSON-LD text
        * A JsonObj object that contains JSON-LD
        * A dictionary that contains JSON-LD

    And turn it into an object that can be tacked onto the end of any JSON object for conversion into RDF

    The base is added back in because @base is ignored in imported and nested contexts -- it must be at the
    root in the object itself.

    :param contexts: Ordered list of contexts to add
    :param base: base to add in (optional)
    :return: aggregated context
    """
    def prune_context_node(ctxt: Union[str, JsonObj]) -> Union[str, JsonObj]:
        return ctxt['@context'] if isinstance(ctxt, JsonObj) and '@context' in ctxt else ctxt

    def to_file_uri(fname: str) -> str:
        return 'file://' + fname

    context_list = []
    for context in [] if contexts is None else [contexts] if not isinstance(contexts, (list, tuple, set)) else contexts:
        if isinstance(context, str):
            # One of filename, URL or json text
            if context.strip().startswith("{"):
                context = loads(context)
            elif '://' not in context:
                context = to_file_uri(context)
        elif not isinstance(context, (JsonObj, str)):
            context = JsonObj(**context)    # dict
        context_list.append(prune_context_node(context))
    if base:
        context_list.append(JsonObj(**{'@base': str(base)}))
    return None if not context_list else \
        JsonObj(**{"@context": context_list[0] if len(context_list) == 1 else context_list})


def parse_import_map(map: Optional[Union[str, Dict[str, str], TextIOWrapper]], base: Optional[str] = None) -> Dict[str, str]:
    """
    Process the import map
    :param map: A map location, the JSON for a map, YAML for a map or an existing dictionary
    :param base: Base location to turn relative locations into absolute
    :return: Import mapp
    """
    if map is None:
        rval = dict()
    elif isinstance(map, TextIOWrapper):
        return parse_import_map(map.read(), base)
    elif isinstance(map, dict):
        rval = map
    elif map.strip().startswith('{'):
        rval = json.loads(map)
    elif '\n' in map or '\r' in map or ' ' in map:
        rval = yaml.load(map)
    else:
        with open(map) as ml:
            return parse_import_map(ml.read(), base)

    if base:
        outmap = dict()
        for k, v in rval.items():
            if ':' not in v:
                v = os.path.join(base, v)
                if ':/' not in v:
                    v = os.path.abspath(v)
            outmap[k] = v
        rval = outmap
    return rval




# id: https://example.com/test44
# description:
# license: https://creativecommons.org/publicdomain/zero/1.0/

import dataclasses
import sys
from typing import Optional, List, Union, Dict, ClassVar, Any
from dataclasses import dataclass
from biolinkml.utils.slot import Slot
from biolinkml.utils.metamodelcore import empty_list, empty_dict, bnode
from biolinkml.utils.yamlutils import YAMLRoot, extended_str, extended_float, extended_int
if sys.version_info < (3, 7, 6):
    from biolinkml.utils.dataclass_extensions_375 import dataclasses_init_fn_with_kwargs
else:
    from biolinkml.utils.dataclass_extensions_376 import dataclasses_init_fn_with_kwargs
from biolinkml.utils.formatutils import camelcase, underscore, sfx
from rdflib import Namespace, URIRef
from biolinkml.utils.curienamespace import CurieNamespace
from biolinkml.utils.metamodelcore import URIorCURIE
from includes.types import Uriorcurie

metamodel_version = "1.4.3"

# Overwrite dataclasses _init_fn to add **kwargs in __init__
dataclasses._init_fn = dataclasses_init_fn_with_kwargs

# Namespaces
META = CurieNamespace('meta', 'https://w3id.org/biolink/biolinkml/')
RDFS = CurieNamespace('rdfs', 'http://example.org/UNKNOWN/rdfs/')
XSD = CurieNamespace('xsd', 'http://www.w3.org/2001/XMLSchema#')
DEFAULT_ = CurieNamespace('', 'https://example.com/test44/')


# Types
class IriType(Uriorcurie):
    """ An IRI """
    type_class_uri = XSD.anyURI
    type_class_curie = "xsd:anyURI"
    type_name = "iri type"
    type_model_uri = URIRef("https://example.com/test44/IriType")


# Class references



@dataclass
class NamedThing(YAMLRoot):
    _inherited_slots: ClassVar[List[str]] = []

    class_class_uri: ClassVar[URIRef] = URIRef("https://example.com/test44/NamedThing")
    class_class_curie: ClassVar[str] = None
    class_name: ClassVar[str] = "named thing"
    class_model_uri: ClassVar[URIRef] = URIRef("https://example.com/test44/NamedThing")

    category: List[Union[str, IriType]] = empty_list()

    def __post_init__(self, **kwargs: Dict[str, Any]):
        if not isinstance(self.category, list) or len(self.category) == 0:
            raise ValueError(f"category must be a non-empty list")
        self.category = [v if isinstance(v, IriType)
                         else IriType(v) for v in self.category]
        super().__post_init__(**kwargs)



# Slots
class slots:
    pass

slots.category = Slot(uri=RDFS.subClassOf, name="category", curie=RDFS.curie('subClassOf'),
                      model_uri=DEFAULT_.category, domain=NamedThing, range=List[Union[str, IriType]])


import os
import unittest
from types import ModuleType

from jsonasobj import as_json

from biolinkml.generators.jsonldcontextgen import ContextGenerator
from biolinkml.generators.pythongen import PythonGenerator
from biolinkml.utils.yamlutils import as_rdf
from tests.test_issues import sourcedir


class Issue80TestCase(unittest.TestCase):

    def header(self, txt: str) -> str:
        return '\n' + ("=" * 20) + f" {txt} " + ("=" * 20)

    def test_issue_80(self):
        """ Make sure that types are generated as part of the output """
        yaml_fname = os.path.join(sourcedir, 'issue_80.yaml')
        python = PythonGenerator(yaml_fname).serialize()
        print(self.header("Python"))
        print(python)
        spec = compile(python, 'test', 'exec')
        module = ModuleType('test')
        exec(spec, module.__dict__)
        example = module.Person("http://example.org/person/17", "Fred Jones", 43)

        # JSON Representation
        print(self.header("JSON"))
        print(as_json(example))

        # Generate a context for this particular model
        print(self.header("Context"))
        context = ContextGenerator(yaml_fname).serialize()
        print(context)

        # RDF Representation
        print(self.header("RDF"))
        print(as_rdf(example, contexts=context).serialize(format="turtle").decode())


if __name__ == '__main__':
    unittest.main()



""" Generate JSONld

"""
import os
from typing import Any, Optional

import click
from jsonasobj import as_json

from biolinkml import METAMODEL_CONTEXT_URI
from biolinkml.meta import ClassDefinitionName, SlotDefinitionName, TypeDefinitionName, \
    ElementName, SlotDefinition, ClassDefinition, TypeDefinition, SubsetDefinitionName, SubsetDefinition
from biolinkml.utils.formatutils import camelcase, underscore
from biolinkml.utils.generator import Generator, shared_arguments
from biolinkml.utils.yamlutils import YAMLRoot


class JSONLDGenerator(Generator):
    generatorname = os.path.basename(__file__)
    generatorversion = "0.0.2"
    valid_formats = ['jsonld']

    def _add_type(self, node: YAMLRoot) -> dict:
        typ = node.__class__.__name__
        node = node.__dict__
        node['type'] = typ
        return node

    def _visit(self, node: Any) -> Optional[Any]:
        if isinstance(node, (YAMLRoot, dict)):
            if isinstance(node, YAMLRoot):
                node = self._add_type(node)
            for k, v in list(node.items()):
                if v:
                    new_v = self._visit(v)
                    if new_v is not None:
                        node[k] = new_v
        elif isinstance(node, list):
            for i in range(0, len(node)):
                new_v = self._visit(node[i])
                if new_v is not None:
                    node[i] = new_v
        elif isinstance(node, set):
            for v in list(node):
                new_v = self._visit(v)
                if new_v is not None:
                    node.remove(v)
                    node.add(new_v)
        elif isinstance(node, ClassDefinitionName):
            return ClassDefinitionName(camelcase(node))
        elif isinstance(node, SlotDefinitionName):
            return SlotDefinitionName(underscore(node))
        elif isinstance(node, TypeDefinitionName):
            return TypeDefinitionName(underscore(node))
        elif isinstance(node, SubsetDefinitionName):
            return SubsetDefinitionName(underscore(node))
        elif isinstance(node, ElementName):
            return ClassDefinitionName(camelcase(node)) if node in self.schema.classes else \
                SlotDefinitionName(underscore(node)) if node in self.schema.slots else \
                SubsetDefinitionName(camelcase(node)) if node in self.schema.subsets else \
                TypeDefinitionName(underscore(node)) if node in self.schema.types else None
        return None

    def adjust_slot(self, slot: SlotDefinition) -> None:
        if slot.range in self.schema.classes:
            slot.range = ClassDefinitionName(camelcase(slot.range))
        elif slot.range in self.schema.slots:
            slot.range = SlotDefinitionName(underscore(slot.range))
        elif slot.range in self.schema.types:
            slot.range = TypeDefinitionName(underscore(slot.range))
        slot.slot_uri = self.namespaces.uri_for(slot.slot_uri)

    def visit_class(self, cls: ClassDefinition) -> bool:
        self._visit(cls)
        cls.class_uri = self.namespaces.uri_for(cls.class_uri)
        # Slot usage is a construction artifact
        cls.slot_usage = []
        return False

    def visit_slot(self, aliased_slot_name: str, slot: SlotDefinition) -> None:
        self._visit(slot)
        self.adjust_slot(slot)

    def visit_type(self, typ: TypeDefinition) -> None:
        self._visit(typ)
        typ.uri = self.namespaces.uri_for(typ.uri)

    def visit_subset(self, ss: SubsetDefinition) -> None:
        self._visit(ss)

    def end_schema(self, context: str = None, **_) -> None:
        self._add_type(self.schema)
        json_obj = self.schema
        base_prefix = self.default_prefix()

        # JSON LD adjusts context reference using '@base'.  If context is supplied and not a URI, generate an
        # absolute URI for it
        if context is None:
            context = [METAMODEL_CONTEXT_URI]
        elif isinstance(context, str):               # Some of the older code doesn't do multiple contexts
            context = [context]
        elif isinstance(context, tuple):
            context = list(context)
        for imp in list(self.loaded.values())[1:]:
            context.append(imp[0] + ".context.jsonld")

        json_obj["@context"] = [context[0]] if len(context) == 1 and not base_prefix else context
        if base_prefix:
            json_obj["@context"].append({'@base': base_prefix})
        # json_obj["@id"] = self.schema.id
        print(as_json(json_obj, indent="  "))


@shared_arguments(JSONLDGenerator)
@click.command()
@click.option("--context", default=METAMODEL_CONTEXT_URI, multiple=True,
              help=f"JSONLD context file (default: {METAMODEL_CONTEXT_URI})")
def cli(yamlfile, **kwargs):
    """ Generate JSONLD file from biolink schema """
    print(JSONLDGenerator(yamlfile, **kwargs).serialize(**kwargs))
